#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use XML::Twig;
use Getopt::Long;
use File::Basename;
use Sleepycat::DbXml 'simple';
use POSIX 'locale_h';
use Unicode::String qw(utf8 latin1);

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'en_meps.dbxml';
my $output_dir = "$root_dir/xml/votes";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <input_text_file>
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically en_meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: en_meps.dbxml
  --output, -o <output_dir>:         output directory
                                     default: $root_dir/xml/votes
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

$theContainer =~ s/s?(?:\.dbxml)?$/s/;

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
            Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");

my $input_file = shift or die "Usage : $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <input_text_file>\n";
my $file_name = fileparse($input_file, qr/\Q.txt\E/);

# Read input file
open INPUT_FILE, $input_file or die "Cannot open $input_file: $!\n";
my @input_lines = <INPUT_FILE>;
close INPUT_FILE;

my @votes     = (); # List of votes
my %this_vote = (); # Structure to collect current vote
my $status    = ''; # Flag to indicate which part is currently parsed
my $voters    = ''; # Concatenated string of voters (needed since a MEP name can be splitted on continuous lines)
my %memoize   = (); # Hash to memoize relation between voting names and MEP identifiers in dbxml
while (<DATA>) {
  next unless /'(.+?)' => \['(.+?)', '(.+?)'\]/o;
  my ($voting, $full, $wiki) = ($1, $2, $3);
  $voting =~ s/\\//o;
  $full =~ s/\\//o;
  $memoize{$voting} = [$full, $wiki];
}

# Parse input file
foreach my $line (@input_lines) {
  chomp $line;

  # Skip page markup
  next if $line =~ /^PV -\d+- PE \d+\.\d+$/o;
  # Skip number of FOR, AGAINST, ABSTENTION votes (but do not skip ABSTENTION votes marked with '0')
  next if $line ne '0' and $line =~ /^\d+$/o;
  # Delete group
  $line =~ s/^(?:ALDE: |GUE\/NGL: |IND\/DEM: |NI: |PPE-DE: |PSE: |UEN: |Verts\/ALE: |EDD: |ELDR: )/, /o;
  # Replace ß for compliance with Europarl directory
  $line =~ s/ß/ss/go;

  # Beginning of a new vote
  if ($line =~ /^\d+\. (.+) - (.+) (\d\d)\/(\d\d)\/(\d\d\d\d) (\d\d):(\d\d):(\d\d)$/o) {
    # End of previous vote (case where no correction of votes have happened)
    if ($status eq 'abstention') {
      $voters =~ s/^, //go;
      $voters =~ s/ ,/,/go;
      $voters =~ s/ $//go;
      push @{$this_vote{VOTE}->{ABSTENTION}}, split(/, /, $voters);
      $voters = '';

      my %vote = %this_vote;
      push @votes, \%vote;
    }

    %this_vote = (
                  SUBJECT => {
                              TEXT       => $1,
                              PART       => $2,
                             },
                  DATE    => {
                              DAY        => $3,
                              MONTH      => $4,
                              YEAR       => $5,
                              HOUR       => $6,
                              MINUTE     => $7,
                              SECOND     => $8,
                             },
                  VOTE    => {
                              FOR        => [],
                              AGAINST    => [],
                              ABSTENTION => [],
                             },
                 );
    $status = 'beginning';
  }

  # Beginning of FOR votes
  elsif ($status eq 'beginning' and ($line eq '+' or
                                   $line =~ /^\d+ PRO - fOR - dAFÜR - Υπέρ - FOR - A FAVOR - POOLT - PUOLESTA - Pour - MELLETTE - A FAVORE - Už - FAVUR - PAR - VOOR -$/o or
                                   $line =~ /^For: \d+$/o)
        ) {
    $status = 'for';
  }

  # Collect FOR votes
  elsif ($status eq 'for') {
    # Skip continuation of beginning of FOR votes
    next if $line =~ /^ZA - A FAVOR - ZA - ZA - JA-RÖSTER$/o;
    # End of FOR votes, beginning of AGAINST votes
    if ($line eq '-' or
        $line =~ /^\d+ PROTI - IMOD - DAGEGEN - Κατά - AGAINST - EN CONTRA - VASTU - VASTAAN - CONTRE - ELLENE - KONTRA - CONTRO -/o or
        $line =~ /^Against: \d+$/o) {
      $status = 'against';
      $voters =~ s/^, //go;
      $voters =~ s/ ,/,/go;
      $voters =~ s/ $//go;
      push @{$this_vote{VOTE}->{FOR}}, split(/, /, $voters);
      $voters = '';
    }
    else {
      $line =~ s/$/ /o unless $line =~ /[ -]$/o;
      $voters .= $line;
    }
  }

  # Collect AGAINST votes
  elsif ($status eq 'against') {
    # Skip continuation of beginning of AGAINST votes
    next if $line =~ /^PRIEš - PRET - TEGEN - PRZECIW - CONTRA - PROTI - PROTI - NEJ-RÖSTER$/o;
    # End of AGAINST votes, beginning of ABSTENTION votes
    if ($line eq '0' or
        $line =~ /^\d+ ZDRŽELI SE HLASOVÁNÍ - HVERKEN ELLER - ENTHALTUNGEN - Απoχές - ABSTENTION - ABSTENCIÓN - ERAPOOLETU - TYHJÄÄ -/o or
        $line =~ /^Abstention: \d+$/o) {
      $status = 'abstention';
      $voters =~ s/^, //go;
      $voters =~ s/ ,/,/go;
      $voters =~ s/ $//go;
      push @{$this_vote{VOTE}->{AGAINST}}, split(/, /, $voters);
      $voters = '';
    }
    else {
      $line =~ s/$/ /o unless $line =~ /[ -]$/o;
      $voters .= $line;
    }
  }

  # Collect ABSTENTION votes
  elsif ($status eq 'abstention') {
    # Skip continuation of beginning of ABSTENTION votes
    next if $line =~ /^ABSTENTION - TARTÓZKODÁS - ASTENSJONI - ASTENSIONI - SUSILAIKĘ - ATTURAS - ONTHOUDINGEN - WSTRZYMUJĄCY SIĘ -$/o or $line =~ /^ABSTENÇÕES - ZDRžALI SA - VzDRžANI - NEDLAGDA RÖSTER$/o;
    # End of ABSTENSION votes, waiting for beginning of a new vote
    if ($line eq 'ПОПРАВКИ В ПОДАДЕНИТЕ ГЛАСОВЕ И НАМЕРЕНИЯ ЗА ГЛАСУВАНЕ - CORRECCIONES E INTENCIONES DE VOTO' or $line eq 'OPRAVY HLASOVÁNÍ A SDĚLENÍ O ÚMYSLU HLASOVAT - STEMMERETTELSER OG -INTENTIONER - BERICHTIGUNGEN') {
      $status = '';
      $voters =~ s/^, //go;
      $voters =~ s/ ,/,/go;
      $voters =~ s/ $//go;
      push @{$this_vote{VOTE}->{ABSTENTION}}, split(/, /, $voters);
      $voters = '';

      my %vote = %this_vote;
      push @votes, \%vote;
    }
    else {
      $line =~ s/$/ /o unless $line =~ /[ -]$/o;
      $voters .= $line;
    }
  }
}

# Collect the last vote
if ($status eq 'abstention') {
  $voters =~ s/^, //go;
  $voters =~ s/ ,/,/go;
  $voters =~ s/ $//go;
  push @{$this_vote{VOTE}->{ABSTENTION}}, split(/, /, $voters);
  #my %vote = %this_vote;
  #push @votes, \%vote;
  push @votes, \%this_vote;
}

# Build XML tree
my $xml = XML::Twig->new(pretty_print => 'indented');
$xml->set_encoding('UTF-8');
my $root = XML::Twig::Elt->new('votes');
$xml->set_root($root);
$root->insert_new_elt('first_child', 'container', 'en_votes.dbxml');

foreach my $vote (@votes) {
  my $vote_elt = $root->insert_new_elt('last_child', 'vote');

  my $subject = $vote_elt->insert_new_elt('last_child', 'subject');
  $subject->insert_new_elt('last_child', 'text', $vote->{SUBJECT}->{TEXT});
  $subject->insert_new_elt('last_child', 'part', $vote->{SUBJECT}->{PART});

  my $date = $vote_elt->insert_new_elt('last_child', 'date');
  $date->insert_new_elt('last_child', 'year', $vote->{DATE}->{YEAR});
  $date->insert_new_elt('last_child', 'month', $vote->{DATE}->{MONTH});
  $date->insert_new_elt('last_child', 'day', $vote->{DATE}->{DAY});
  $date->insert_new_elt('last_child', 'hour', $vote->{DATE}->{HOUR});
  $date->insert_new_elt('last_child', 'minute', $vote->{DATE}->{MINUTE});
  $date->insert_new_elt('last_child', 'second', $vote->{DATE}->{SECOND});

  my $result = $vote_elt->insert_new_elt('last_child', 'result');
  foreach my $mep (@{$vote->{VOTE}->{FOR}}) {
    # Search for MEP's name in dbxml
    my ($name, $wiki_name) = mep_name($mep);
    $name = $mep unless $name;
    my $mep_elt = $result->insert_new_elt('last_child', 'mep');
    $mep_elt->insert_new_elt('last_child', 'name', $name);
    $mep_elt->insert_new_elt('last_child', 'dbxmlid', $wiki_name) if $wiki_name;
    $mep_elt->insert_new_elt('last_child', 'choice', 'for');
  }
  foreach my $mep (@{$vote->{VOTE}->{AGAINST}}) {
    # Search for MEP's name in dbxml
    my ($name, $wiki_name) = mep_name($mep);
    $name = $mep unless $name;
    my $mep_elt = $result->insert_new_elt('last_child', 'mep');
    $mep_elt->insert_new_elt('last_child', 'name', $name);
    $mep_elt->insert_new_elt('last_child', 'dbxmlid', $wiki_name) if $wiki_name;
    $mep_elt->insert_new_elt('last_child', 'choice', 'against');
  }
  foreach my $mep (@{$vote->{VOTE}->{ABSTENTION}}) {
    # Search for MEP's name in dbxml
    my ($name, $wiki_name) = mep_name($mep);
    $name = $mep unless $name;
    my $mep_elt = $result->insert_new_elt('last_child', 'mep');
    $mep_elt->insert_new_elt('last_child', 'name', $name);
    $mep_elt->insert_new_elt('last_child', 'dbxmlid', $wiki_name) if $wiki_name;
    $mep_elt->insert_new_elt('last_child', 'choice', 'abstention');
  }
}

my $xml_location = "$output_dir/$file_name.xml";
open XML, ">$xml_location" or die "Cannot write in file $xml_location: $!\n";
$xml->flush(\*XML);
close XML or die "Error when closing $xml_location: $!\n";

sub mep_name {
  my $voting_name = shift;

  # Memoization
  if (exists $memoize{$voting_name}) {
    return wantarray ? ($memoize{$voting_name}->[0], $memoize{$voting_name}->[1]) : $memoize{$voting_name}->[1];
  }
  my $memoize_name = $voting_name;

  # French particule should be in lower case
  $voting_name =~ s/^D([e'])\b/d$1/go;
  # Name after 'Mc' is in lower case
  $voting_name =~ s/Mc(.)/Mc\l$1/go;
  # E, Del, Den, Dos, I, In, Of, Van, Von, T, Y and Zu particules are in upper case
  $voting_name =~ s/(^| |')(e|del|den|dos|i|in|of|t|van|von|y|zu)( |'|$)/$1\u$2$3/go;
  # Escape quote
  $voting_name =~ s/'/&apos;/go;

  my $query = "collection('dbxml:/$theContainer.dbxml')/politician/infos/name[last='$voting_name']";
  my $results = $theMgr->query($query);
  unless ($results and $results->size == 1) {
    $query = "collection('dbxml:/$theContainer.dbxml')/politician/infos/name[concat(last, ' ', first)='$voting_name']";
    $results = $theMgr->query($query);
  }

  unless ($results and $results->size == 1) {
    warn "Not found voting_name=<$memoize_name> searched=<$voting_name> result size=<".($results ? $results->size : 'undef').">\n";
    $memoize{$memoize_name} = [$memoize_name, undef];
    return wantarray ? ($memoize{$memoize_name}->[0], $memoize{$memoize_name}->[1]) : $memoize{$memoize_name}->[1];
  }

  my $value = new XmlValue;
  $results->next($value);
  (my $full_name) = ($value =~ /<full>([^>]+)<\/full>/o);
  (my $wiki_name) = ($value =~ /<wiki>([^>]+)<\/wiki>/o);

  $memoize{$memoize_name} = [$full_name, $wiki_name];
  print "'$memoize_name' => ['$full_name', '$wiki_name'],\n"; 

  return wantarray ? ($memoize{$memoize_name}->[0], $memoize{$memoize_name}->[1]) : $memoize{$memoize_name}->[1];
}

__DATA__
'Adamou' => ['Adamos ADAMOU', 'AdamosAdamou']
'Agnoletto' => ['Vittorio AGNOLETTO', 'VittorioAgnoletto']
'Aita' => ['Vincenzo AITA', 'VincenzoAita']
'Albertini' => ['Gabriele ALBERTINI', 'GabrieleAlbertini']
'Alvaro' => ['Alexander ALVARO', 'AlexanderAlvaro']
'Anastase' => ['Roberta Alma ANASTASE', 'RobertaAlmaAnastase']
'Andersson' => ['Jan ANDERSSON', 'JanAndersson']
'Andrejevs' => ['Georgs ANDREJEVS', 'GeorgsAndrejevs']
'Andrikienė' => ['Laima Liucija ANDRIKIENĖ', 'LaimaLiucijaAndrikiene']
'Arif' => ['Kader ARIF', 'KaderArif']
'Ashworth' => ['Richard James ASHWORTH', 'RichardJamesAshworth']
'Assis' => ['Francisco ASSIS', 'FranciscoAssis']
'Atkins' => ['Sir Robert ATKINS', 'SirRobertAtkins']
'Attard-Montalto' => ['John ATTARD-MONTALTO', 'JohnAttardMontalto']
'Attwooll' => ['Elspeth ATTWOOLL', 'ElspethAttwooll']
'Aubert' => ['Marie-Hélène AUBERT', 'MarieHeleneAubert']
'Audy' => ['Jean-Pierre AUDY', 'JeanPierreAudy']
'Auken' => ['Margrete AUKEN', 'MargreteAuken']
'Ayala Sender' => ['Inés AYALA SENDER', 'InesAyalaSender']
'Aylward' => ['Liam AYLWARD', 'LiamAylward']
'Ayuso' => ['Pilar AYUSO', 'PilarAyuso']
'Baco' => ['Peter BACO', 'PeterBaco']
'Badia i Cutchet' => ['Maria BADIA i CUTCHET', 'MariaBadiaICutchet']
'Baeva' => ['Mariela Velichkova BAEVA', 'MarielaVelichkovaBaeva']
'Barsi-Pataky' => ['Etelka BARSI-PATAKY', 'EtelkaBarsiPataky']
'Barón Crespo' => ['Enrique BARÓN CRESPO', 'EnriqueBaronCrespo']
'Battilocchio' => ['Alessandro BATTILOCCHIO', 'AlessandroBattilocchio']
'Batzeli' => ['Katerina BATZELI', 'KaterinaBatzeli']
'Bauer' => ['Edit BAUER', 'EditBauer']
'Beaupuy' => ['Jean Marie BEAUPUY', 'JeanMarieBeaupuy']
'Beazley' => ['Christopher BEAZLEY', 'ChristopherBeazley']
'Becsey' => ['Zsolt László BECSEY', 'ZsoltLaszloBecsey']
'Beer' => ['Angelika BEER', 'AngelikaBeer']
'Belder' => ['Bastiaan BELDER', 'BastiaanBelder']
'Belet' => ['Ivo BELET', 'IvoBelet']
'Belohorská' => ['Irena BELOHORSKÁ', 'IrenaBelohorska']
'Bennahmias' => ['Jean-Luc BENNAHMIAS', 'JeanLucBennahmias']
'Berend' => ['Rolf BEREND', 'RolfBerend']
'Berlato' => ['Sergio BERLATO', 'SergioBerlato']
'Berman' => ['Thijs BERMAN', 'ThijsBerman']
'Berès' => ['Pervenche BERÈS', 'PervencheBeres']
'Beňová' => ['Monika BEŇOVÁ', 'MonikaBenova']
'Bielan' => ['Adam BIELAN', 'AdamBielan']
'Binev' => ['Slavi BINEV', 'SlaviBinev']
'Birutis' => ['Šarūnas BIRUTIS', 'SarunasBirutis']
'Blokland' => ['Johannes BLOKLAND', 'JohannesBlokland']
'Bobošíková' => ['Jana BOBOŠÍKOVÁ', 'JanaBobosikova']
'Bodu' => ['Sebastian Valentin BODU', 'SebastianValentinBodu']
'Bono' => ['Guy BONO', 'GuyBono']
'Booth' => ['Graham BOOTH', 'GrahamBooth']
'Borghezio' => ['Mario BORGHEZIO', 'MarioBorghezio']
'Bourzai' => ['Bernadette BOURZAI', 'BernadetteBourzai']
'Bowis' => ['John BOWIS', 'JohnBowis']
'Bowles' => ['Sharon BOWLES', 'SharonBowles']
'Bozkurt' => ['Emine BOZKURT', 'EmineBozkurt']
'Boştinaru' => ['Victor BOŞTINARU', 'VictorBostinaru']
'Bradbourn' => ['Philip BRADBOURN', 'PhilipBradbourn']
'Braghetto' => ['Iles BRAGHETTO', 'IlesBraghetto']
'Brejc' => ['Mihael BREJC', 'MihaelBrejc']
'Brepoels' => ['Frieda BREPOELS', 'FriedaBrepoels']
'Brie' => ['André BRIE', 'AndreBrie']
'Brok' => ['Elmar BROK', 'ElmarBrok']
'Budreikaitė' => ['Danutė BUDREIKAITĖ', 'DanuteBudreikaite']
'Bulfon' => ['Wolfgang BULFON', 'WolfgangBulfon']
'Bullmann' => ['Udo BULLMANN', 'UdoBullmann']
'Bulzesc' => ['Nicodim BULZESC', 'NicodimBulzesc']
'Burke' => ['Colm BURKE', 'ColmBurke']
'Bushill-Matthews' => ['Philip BUSHILL-MATTHEWS', 'PhilipBushillMatthews']
'Busk' => ['Niels BUSK', 'NielsBusk']
'Buşoi' => ['Cristian Silviu BUŞOI', 'CristianSilviuBusoi']
'Busquin' => ['Philippe BUSQUIN', 'PhilippeBusquin']
'Busuttil' => ['Simon BUSUTTIL', 'SimonBusuttil']
'Buzek' => ['Jerzy BUZEK', 'JerzyBuzek']
'Böge' => ['Reimer BÖGE', 'ReimerBoge']
'Bösch' => ['Herbert BÖSCH', 'HerbertBosch']
'Březina' => ['Jan BŘEZINA', 'JanBrezina']
'Cabrnoch' => ['Milan CABRNOCH', 'MilanCabrnoch']
'Callanan' => ['Martin CALLANAN', 'MartinCallanan']
'Camre' => ['Mogens CAMRE', 'MogensCamre']
'Capoulas Santos' => ['Luis Manuel CAPOULAS SANTOS', 'LuisManuelCapoulasSantos']
'Cappato' => ['Marco CAPPATO', 'MarcoCappato']
'Carlotti' => ['Marie-Arlette CARLOTTI', 'MarieArletteCarlotti']
'Carlshamre' => ['Maria CARLSHAMRE', 'MariaRobsahm']
'Carnero González' => ['Carlos CARNERO GONZÁLEZ', 'CarlosCarneroGonzalez']
'Carollo' => ['Giorgio CAROLLO', 'GiorgioCarollo']
'Casa' => ['David CASA', 'DavidCasa']
'Casaca' => ['Paulo CASACA', 'PauloCasaca']
'Casini' => ['Carlo CASINI', 'CarloCasini']
'Caspary' => ['Daniel CASPARY', 'DanielCaspary']
'Castex' => ['Françoise CASTEX', 'FrancoiseCastex']
'Cavada' => ['Jean-Marie CAVADA', 'JeanMarieCavada']
'Cederschiöld' => ['Charlotte CEDERSCHIÖLD', 'CharlotteCederschiold']
'Cercas' => ['Alejandro CERCAS', 'AlejandroCercas']
'Chatzimarkakis' => ['Jorgo CHATZIMARKAKIS', 'JorgoChatzimarkakis']
'Chichester' => ['Giles CHICHESTER', 'GilesChichester']
'Chiesa' => ['Giulietto CHIESA', 'GiuliettoChiesa']
'Chmielewski' => ['Zdzisław Kazimierz CHMIELEWSKI', 'ZdzislawKazimierzChmielewski']
'Christensen' => ['Ole CHRISTENSEN', 'OleChristensen']
'Chruszcz' => ['Sylwester CHRUSZCZ', 'SylwesterChruszcz']
'Chukolov' => ['Desislav CHUKOLOV', 'DesislavChukolov']
'Claeys' => ['Philip CLAEYS', 'PhilipClaeys']
'Clark' => ['Derek Roland CLARK', 'DerekRolandClark']
'Cocilovo' => ['Luigi COCILOVO', 'LuigiCocilovo']
'Coelho' => ['Carlos COELHO', 'CarlosCoelho']
'Cohn-Bendit' => ['Daniel COHN-BENDIT', 'DanielCohnBendit']
'Corbett' => ['Richard CORBETT', 'RichardCorbett']
'Corbey' => ['Dorette CORBEY', 'DoretteCorbey']
'Corda' => ['Giovanna CORDA', 'GiovannaCorda']
'Costa' => ['Paolo COSTA', 'PaoloCosta']
'Cottigny' => ['Jean Louis COTTIGNY', 'JeanLouisCottigny']
'Cramer' => ['Michael CRAMER', 'MichaelCramer']
'Creţu Corina' => ['Corina CREŢU', 'CorinaCretu']
'Creţu Gabriela' => ['Gabriela CREŢU', 'GabrielaCretu']
'Czarnecki Ryszard' => ['Ryszard CZARNECKI', 'RyszardCzarnecki']
'Daul' => ['Joseph DAUL', 'JosephDaul']
'David' => ['Dragoş Florin DAVID', 'DragosFlorinDavid']
'Davies' => ['Chris DAVIES', 'ChrisDavies']
'De Blasio' => ['Antonio DE BLASIO', 'AntoniodeBlasio']
'De Keyser' => ['Véronique DE KEYSER', 'VeroniquedeKeyser']
'De Lange' => ['Esther de LANGE', 'EstherdeLange']
'De Rossa' => ['Proinsias DE ROSSA', 'ProinsiasdeRossa']
'De Sarnez' => ['Marielle DE SARNEZ', 'MarielledeSarnez']
'De Veyrac' => ['Christine DE VEYRAC', 'ChristinedeVeyrac']
'De Vits' => ['Mia DE VITS', 'MiadeVits']
'Degutis' => ['Arūnas DEGUTIS', 'ArunasDegutis']
'Dehaene' => ['Jean-Luc DEHAENE', 'JeanLucDehaene']
'Demetriou' => ['Panayiotis DEMETRIOU', 'PanayiotisDemetriou']
'Deprez' => ['Gérard DEPREZ', 'GerardDeprez']
'Descamps' => ['Marie-Hélène DESCAMPS', 'MarieHeleneDescamps']
'Dess' => ['Albert DESS', 'AlbertDess']
'Dillen' => ['Koenraad DILLEN', 'KoenraadDillen']
'Dimitrakopoulos' => ['Giorgos DIMITRAKOPOULOS', 'GiorgosDimitrakopoulos']
'Dičkutė' => ['Jolanta DIČKUTĖ', 'JolantaDickute']
'Dobolyi' => ['Alexandra DOBOLYI', 'AlexandraDobolyi']
'Dombrovskis' => ['Valdis DOMBROVSKIS', 'ValdisDombrovskis']
'Donnici' => ['Beniamino DONNICI', 'BeniaminoDonnici']
'Doorn' => ['Bert DOORN', 'BertDoorn']
'Douay' => ['Brigitte DOUAY', 'BrigitteDouay']
'Dover' => ['Den DOVER', 'DenDover']
'Doyle' => ['Avril DOYLE', 'AvrilDoyle']
'Drčar Murko' => ['Mojca DRČAR MURKO', 'MojcaDrcarMurko']
'Duchoň' => ['Petr DUCHOŇ', 'PetrDuchon']
'Duff' => ['Andrew DUFF', 'AndrewDuff']
'Duka-Zólyomi' => ['Árpád DUKA-ZÓLYOMI', 'ArpadDukaZolyomi']
'Dumitriu' => ['Constantin DUMITRIU', 'ConstantinDumitriu']
'Díaz de Mera García Consuegra' => ['Agustín DÍAZ DE MERA GARCÍA CONSUEGRA', 'AgustinDiazdeMeraGarciaConsuegra']
'Ebner' => ['Michl EBNER', 'MichlEbner']
'Ehler' => ['Christian EHLER', 'ChristianEhler']
'Ek' => ['Lena EK', 'LenaEk']
'El Khadraoui' => ['Saïd EL KHADRAOUI', 'SaidElKhadraoui']
'Elles' => ['James ELLES', 'JamesElles']
'Estrela' => ['Edite ESTRELA', 'EditeEstrela']
'Ettl' => ['Harald ETTL', 'HaraldEttl']
'Evans Jill' => ['Jill EVANS', 'JillEvans']
'Evans Robert' => ['Robert EVANS', 'RobertEvans']
'Fajmon' => ['Hynek FAJMON', 'HynekFajmon']
'Falbr' => ['Richard FALBR', 'RichardFalbr']
'Farage' => ['Nigel FARAGE', 'NigelFarage']
'Fatuzzo' => ['Carlo FATUZZO', 'CarloFatuzzo']
'Fazakas' => ['Szabolcs FAZAKAS', 'SzabolcsFazakas']
'Fernandes' => ['Emanuel Jardim FERNANDES', 'EmanuelJardimFernandes']
'Ferrari' => ['Francesco FERRARI', 'FrancescoFerrari']
'Ferreira Anne' => ['Anne FERREIRA', 'AnneFerreira']
'Ferreira Elisa' => ['Elisa FERREIRA', 'ElisaFerreira']
'Figueiredo' => ['Ilda FIGUEIREDO', 'IldaFigueiredo']
'Filip' => ['Petru FILIP', 'PetruFilip']
'Fjellner' => ['Christofer FJELLNER', 'ChristoferFjellner']
'Flasarová' => ['Věra FLASAROVÁ', 'VeraFlasarova']
'Flautre' => ['Hélène FLAUTRE', 'HeleneFlautre']
'Foglietta' => ['Alessandro FOGLIETTA', 'AlessandroFoglietta']
'Foltyn-Kubicka' => ['Hanna FOLTYN-KUBICKA', 'HannaFoltynKubicka']
'Fontaine' => ['Nicole FONTAINE', 'NicoleFontaine']
'Ford' => ['Glyn FORD', 'GlynFord']
'Fourtou' => ['Janelly FOURTOU', 'JanellyFourtou']
'Fouré' => ['Brigitte FOURÉ', 'BrigitteFoure']
'Fraga Estévez' => ['Carmen FRAGA ESTÉVEZ', 'CarmenFragaEstevez']
'Fraile Cantón' => ['Juan FRAILE CANTÓN', 'JuanFraileCanton']
'França' => ['Armando FRANÇA', 'ArmandoFranca']
'Frassoni' => ['Monica FRASSONI', 'MonicaFrassoni']
'Friedrich' => ['Ingo FRIEDRICH', 'IngoFriedrich']
'Färm' => ['Göran FÄRM', 'GoranFarm']
'Gacek' => ['Urszula GACEK', 'UrszulaGacek']
'Gahler' => ['Michael GAHLER', 'MichaelGahler']
'Galeote' => ['Gerardo GALEOTE', 'GerardoGaleote']
'Garcés Ramón' => ['Vicente Miguel GARCÉS RAMÓN', 'VicenteMiguelGarcesRamon']
'García Pérez' => ['Iratxe GARCÍA PÉREZ', 'IratxeGarciaPerez']
'García-Margallo y Marfil' => ['José Manuel GARCÍA-MARGALLO Y MARFIL', 'JoseManuelGarciaMargalloYMarfil']
'Gaubert' => ['Patrick GAUBERT', 'PatrickGaubert']
'Gauzès' => ['Jean-Paul GAUZÈS', 'JeanPaulGauzes']
'Gaľa' => ['Milan GAĽA', 'MilanGala']
'Gebhardt' => ['Evelyne GEBHARDT', 'EvelyneGebhardt']
'Gentvilas' => ['Eugenijus GENTVILAS', 'EugenijusGentvilas']
'Georgiou' => ['Georgios GEORGIOU', 'GeorgiosGeorgiou']
'Geremek' => ['Bronisław GEREMEK', 'BronislawGeremek']
'Geringer de Oedenberg' => ['Lidia Joanna GERINGER de OEDENBERG', 'LidiaJoannaGeringerdeOedenberg']
'Gewalt' => ['Roland GEWALT', 'RolandGewalt']
'Gibault' => ['Claire GIBAULT', 'ClaireGibault']
'Gierek' => ['Adam GIEREK', 'AdamGierek']
'Giertych' => ['Maciej Marian GIERTYCH', 'MaciejMarianGiertych']
'Gklavakis' => ['Ioannis GKLAVAKIS', 'IoannisGklavakis']
'Glante' => ['Norbert GLANTE', 'NorbertGlante']
'Glattfelder' => ['Béla GLATTFELDER', 'BelaGlattfelder']
'Goebbels' => ['Robert GOEBBELS', 'RobertGoebbels']
'Goepel' => ['Lutz GOEPEL', 'LutzGoepel']
'Golik' => ['Bogdan GOLIK', 'BogdanGolik']
'Gollnisch' => ['Bruno GOLLNISCH', 'BrunoGollnisch']
'Gomes' => ['Ana Maria GOMES', 'AnaMariaGomes']
'Gomolka' => ['Alfred GOMOLKA', 'AlfredGomolka']
'Gottardi' => ['Donata GOTTARDI', 'DonataGottardi']
'Goudin' => ['Hélène GOUDIN', 'HeleneGoudin']
'Grabowska' => ['Genowefa GRABOWSKA', 'GenowefaGrabowska']
'Grabowski' => ['Dariusz Maciej GRABOWSKI', 'DariuszMaciejGrabowski']
'Graefe zu Baringdorf' => ['Friedrich-Wilhelm GRAEFE zu BARINGDORF', 'FriedrichWilhelmGraefeZuBaringdorf']
'Grau i Segú' => ['Martí GRAU i SEGÚ', 'MartiGrauISegu']
'Graça Moura' => ['Vasco GRAÇA MOURA', 'VascoGracaMoura']
'Grech' => ['Louis GRECH', 'LouisGrech']
'Griesbeck' => ['Nathalie GRIESBECK', 'NathalieGriesbeck']
'Groote' => ['Matthias GROOTE', 'MatthiasGroote']
'Grosch' => ['Mathieu GROSCH', 'MathieuGrosch']
'Grossetête' => ['Françoise GROSSETÊTE', 'FrancoiseGrossetete']
'Gruber' => ['Lilli GRUBER', 'LilliGruber']
'Grässle' => ['Ingeborg GRÄSSLE', 'IngeborgGrassle']
'Gröner' => ['Lissy GRÖNER', 'LissyGroner']
'Guardans Cambó' => ['Ignasi GUARDANS CAMBÓ', 'IgnasiGuardansCambo']
'Guellec' => ['Ambroise GUELLEC', 'AmbroiseGuellec']
'Guerreiro' => ['Pedro GUERREIRO', 'PedroGuerreiro']
'Guidoni' => ['Umberto GUIDONI', 'UmbertoGuidoni']
'Gurmai' => ['Zita GURMAI', 'ZitaGurmai']
'Guy-Quint' => ['Catherine GUY-QUINT', 'CatherineGuyQuint']
'Gyürk' => ['András GYÜRK', 'AndrasGyurk']
'Gál' => ['Kinga GÁL', 'KingaGal']
'Hall' => ['Fiona HALL', 'FionaHall']
'Hammerstein' => ['David HAMMERSTEIN', 'DavidHammerstein']
'Hammerstein Mintz' => ['David HAMMERSTEIN', 'DavidHammerstein']
'Hamon' => ['Benoît HAMON', 'BenoitHamon']
'Handzlik' => ['Małgorzata HANDZLIK', 'MalgorzataHandzlik']
'Hannan' => ['Daniel HANNAN', 'DanielHannan']
'Harbour' => ['Malcolm HARBOUR', 'MalcolmHarbour']
'Harms' => ['Rebecca HARMS', 'RebeccaHarms']
'Hasse Ferreira' => ['Joel HASSE FERREIRA', 'JoelHasseFerreira']
'Hassi' => ['Satu HASSI', 'SatuHassi']
'Haug' => ['Jutta HAUG', 'JuttaHaug']
'Heaton-Harris' => ['Christopher HEATON-HARRIS', 'ChristopherHeatonHarris']
'Hedh' => ['Anna HEDH', 'AnnaHedh']
'Hegyi' => ['Gyula HEGYI', 'GyulaHegyi']
'Helmer' => ['Roger HELMER', 'RogerHelmer']
'Hennicot-Schoepges' => ['Erna HENNICOT-SCHOEPGES', 'ErnaHennicotSchoepges']
'Herczog' => ['Edit HERCZOG', 'EditHerczog']
'Herranz García' => ['Esther HERRANZ GARCÍA', 'EstherHerranzGarcia']
'Hieronymi' => ['Ruth HIERONYMI', 'RuthHieronymi']
'Higgins' => ['Jim HIGGINS', 'JimHiggins']
'Holm' => ['Jens HOLM', 'JensHolm']
'Honeyball' => ['Mary HONEYBALL', 'MaryHoneyball']
'Hoppenstedt' => ['Karsten Friedrich HOPPENSTEDT', 'KarstenFriedrichHoppenstedt']
'Horáček' => ['Milan HORÁČEK', 'MilanHoracek']
'Hudacký' => ['Ján HUDACKÝ', 'JanHudacky']
'Hudghton' => ['Ian HUDGHTON', 'IanHudghton']
'Hughes' => ['Stephen HUGHES', 'StephenHughes']
'Hutchinson' => ['Alain HUTCHINSON', 'AlainHutchinson']
'Hybášková' => ['Jana HYBÁŠKOVÁ', 'JanaHybaskova']
'Hyusmenova' => ['Filiz Hakaeva HYUSMENOVA', 'FilizHakaevaHyusmenova']
'Hänsch' => ['Klaus HÄNSCH', 'KlausHansch']
'Hénin' => ['Jacky HÉNIN', 'JackyHenin']
'Hökmark' => ['Gunnar HÖKMARK', 'GunnarHokmark']
'Iacob-Ridzi' => ['Monica Maria IACOB-RIDZI', 'MonicaMariaIacobRidzi']
'Ibrisagic' => ['Anna IBRISAGIC', 'AnnaIbrisagic']
'Iotova' => ['Iliana Malinova IOTOVA', 'IlianaMalinovaIotova']
'Isler Béguin' => ['Marie Anne ISLER BÉGUIN', 'MarieAnneIslerBeguin']
'Jackson' => ['Caroline JACKSON', 'CarolineJackson']
'Janowski' => ['Mieczysław Edmund JANOWSKI', 'MieczyslawEdmundJanowski']
'Jarzembowski' => ['Georg JARZEMBOWSKI', 'GeorgJarzembowski']
'Jałowiecki' => ['Stanisław JAŁOWIECKI', 'StanislawJalowiecki']
'Jeggle' => ['Elisabeth JEGGLE', 'ElisabethJeggle']
'Jeleva' => ['Rumiana JELEVA', 'RumianaJeleva']
'Jensen' => ['Anne E. JENSEN', 'AnneEJensen']
'Jonckheer' => ['Pierre JONCKHEER', 'PierreJonckheer']
'Jordan Cizelj' => ['Romana JORDAN CIZELJ', 'RomanaJordanCizelj']
'Jouye de Grandmaison' => ['Madeleine JOUYE DE GRANDMAISON', 'MadeleineJouyedeGrandmaison']
'Juknevičienė' => ['Ona JUKNEVIČIENĖ', 'OnaJukneviciene']
'Járóka' => ['Lívia JÁRÓKA', 'LiviaJaroka']
'Jäätteenmäki' => ['Anneli JÄÄTTEENMÄKI', 'AnneliJaatteenmaki']
'Jöns' => ['Karin JÖNS', 'KarinJons']
'Kacin' => ['Jelko KACIN', 'JelkoKacin']
'Kallenbach' => ['Gisela KALLENBACH', 'GiselaKallenbach']
'Karas' => ['Othmar KARAS', 'OthmarKaras']
'Karim' => ['Sajjad KARIM', 'SajjadKarim']
'Kasoulides' => ['Ioannis KASOULIDES', 'IoannisKasoulides']
'Kaufmann' => ['Sylvia-Yvonne KAUFMANN', 'SylviaYvonneKaufmann']
'Kauppi' => ['Piia-Noora KAUPPI', 'PiiaNooraKauppi']
'Kazak' => ['Metin KAZAK', 'MetinKazak']
'Kelam' => ['Tunne KELAM', 'TunneKelam']
'Kilroy-Silk' => ['Robert KILROY-SILK', 'RobertKilroySilk']
'Kindermann' => ['Heinz KINDERMANN', 'HeinzKindermann']
'Kinnock' => ['Glenys KINNOCK', 'GlenysKinnock']
'Kirilov' => ['Evgeni KIRILOV', 'EvgeniKirilov']
'Kirkhope' => ['Timothy KIRKHOPE', 'TimothyKirkhope']
'Klamt' => ['Ewa KLAMT', 'EwaKlamt']
'Klass' => ['Christa KLASS', 'ChristaKlass']
'Knapman' => ['Roger KNAPMAN', 'RogerKnapman']
'Koch' => ['Dieter-Lebrecht KOCH', 'DieterLebrechtKoch']
'Koch-Mehrin' => ['Silvana KOCH-MEHRIN', 'SilvanaKochMehrin']
'Kohlíček' => ['Jaromír KOHLÍČEK', 'JaromirKohlicek']
'Konrad' => ['Christoph KONRAD', 'ChristophKonrad']
'Koppa' => ['Maria Eleni KOPPA', 'MariaEleniKoppa']
'Korhola' => ['Eija-Riitta KORHOLA', 'EijaRiittaKorhola']
'Koterec' => ['Miloš KOTEREC', 'MilosKoterec']
'Kozlík' => ['Sergej KOZLÍK', 'SergejKozlik']
'Krasts' => ['Guntars KRASTS', 'GuntarsKrasts']
'Kratsa-Tsagaropoulou' => ['Rodi KRATSA-TSAGAROPOULOU', 'RodiKratsaTsagaropoulou']
'Krehl' => ['Constanze Angela KREHL', 'ConstanzeAngelaKrehl']
'Kreissl-Dörfler' => ['Wolfgang KREISSL-DÖRFLER', 'WolfgangKreisslDorfler']
'Kristovskis' => ['Ģirts Valdis KRISTOVSKIS', 'GirtsValdisKristovskis']
'Krupa' => ['Urszula KRUPA', 'UrszulaKrupa']
'Kuc' => ['Wiesław Stefan KUC', 'WieslawStefanKuc']
'Kusstatscher' => ['Sepp KUSSTATSCHER', 'SeppKusstatscher']
'Kułakowski' => ['Jan Jerzy KUŁAKOWSKI', 'JanJerzyKulakowski']
'Kušķis' => ['Aldis KUŠĶIS', 'AldisKuskis']
'Kuźmiuk' => ['Zbigniew Krzysztof KUŹMIUK', 'ZbigniewKrzysztofKuzmiuk']
'Kósáné Kovács' => ['Magda KÓSÁNÉ KOVÁCS', 'MagdaKosaneKovacs']
'Lagendijk' => ['Joost LAGENDIJK', 'JoostLagendijk']
'Laignel' => ['André LAIGNEL', 'AndreLaignel']
'Lamassoure' => ['Alain LAMASSOURE', 'AlainLamassoure']
'Lambert' => ['Jean LAMBERT', 'JeanLambert']
'Lambrinidis' => ['Stavros LAMBRINIDIS', 'StavrosLambrinidis']
'Lambsdorff' => ['Graf Alexander LAMBSDORFF', 'AlexanderGrafLambsdorff']
'Landsbergis' => ['Vytautas LANDSBERGIS', 'VytautasLandsbergis']
'Lang' => ['Carl LANG', 'CarlLang']
'Langen' => ['Werner LANGEN', 'WernerLangen']
'Langendries' => ['Raymond LANGENDRIES', 'RaymondLangendries']
'Laperrouze' => ['Anne LAPERROUZE', 'AnneLaperrouze']
'Lauk' => ['Kurt Joachim LAUK', 'KurtJoachimLauk']
'Le Foll' => ['Stéphane LE FOLL', 'StephaneLeFoll']
'Le Pen Jean-Marie' => ['Jean-Marie LE PEN', 'JeanMarieLePen']
'Lebech' => ['Johannes LEBECH', 'JohannesLebech']
'Lechner' => ['Kurt LECHNER', 'KurtLechner']
'Lefrançois' => ['Roselyne LEFRANÇOIS', 'RoselyneLefrancois']
'Lehideux' => ['Bernard LEHIDEUX', 'BernardLehideux']
'Lehne' => ['Klaus-Heiner LEHNE', 'KlausHeinerLehne']
'Lehtinen' => ['Lasse LEHTINEN', 'LasseLehtinen']
'Leichtfried' => ['Jörg LEICHTFRIED', 'JorgLeichtfried']
'Leinen' => ['Jo LEINEN', 'JoLeinen']
'Lewandowski' => ['Janusz LEWANDOWSKI', 'JanuszLewandowski']
'Liberadzki' => ['Bogusław LIBERADZKI', 'BoguslawLiberadzki']
'Libicki' => ['Marcin LIBICKI', 'MarcinLibicki']
'Lichtenberger' => ['Eva LICHTENBERGER', 'EvaLichtenberger']
'Lienemann' => ['Marie-Noëlle LIENEMANN', 'MarieNoelleLienemann']
'Liese' => ['Peter LIESE', 'PeterLiese']
'Liotard' => ['Kartika Tamara LIOTARD', 'KartikaTamaraLiotard']
'Lipietz' => ['Alain LIPIETZ', 'AlainLipietz']
'Locatelli' => ['Pia Elda LOCATELLI', 'PiaEldaLocatelli']
'Louis' => ['Patrick LOUIS', 'PatrickLouis']
'Lucas' => ['Caroline LUCAS', 'CarolineLucas']
'Ludford' => ['Baroness Sarah LUDFORD', 'BaronessSarahLudford']
'Lulling' => ['Astrid LULLING', 'AstridLulling']
'Lundgren' => ['Nils LUNDGREN', 'NilsLundgren']
'Luque Aguilar' => ['Florencio LUQUE AGUILAR', 'FlorencioLuqueAguilar']
'Lynne' => ['Elizabeth LYNNE', 'ElizabethLynne']
'Lyubcheva' => ['Marusya Ivanova LYUBCHEVA', 'MarusyaIvanovaLyubcheva']
'López-Istúriz White' => ['Antonio LÓPEZ-ISTÚRIZ WHITE', 'AntonioLopezIsturizWhite']
'Maaten' => ['Jules MAATEN', 'JulesMaaten']
'Madeira' => ['Jamila MADEIRA', 'JamilaMadeira']
'Maldeikis' => ['Eugenijus MALDEIKIS', 'EugenijusMaldeikis']
'Manders' => ['Toine MANDERS', 'ToineManders']
'Mann Erika' => ['Erika MANN', 'ErikaMann']
'Mann Thomas' => ['Thomas MANN', 'ThomasMann']
'Marinescu' => ['Marian-Jean MARINESCU', 'MarianJeanMarinescu']
'Marques' => ['Sérgio MARQUES', 'SergioMarques']
'Martens' => ['Maria MARTENS', 'MariaMartens']
'Martin David' => ['David MARTIN', 'DavidMartin']
'Martin Hans-Peter' => ['Hans-Peter MARTIN', 'HansPeterMartin']
'Martinez' => ['Jean-Claude MARTINEZ', 'JeanClaudeMartinez']
'Martínez Martínez' => ['Miguel Angel MARTÍNEZ MARTÍNEZ', 'MiguelAngelMartinezMartinez']
'Masiel' => ['Jan Tadeusz MASIEL', 'JanTadeuszMasiel']
'Matsakis' => ['Marios MATSAKIS', 'MariosMatsakis']
'Mauro' => ['Mario MAURO', 'MarioMauro']
'Mavrommatis' => ['Manolis MAVROMMATIS', 'ManolisMavrommatis']
'Mayer' => ['Hans-Peter MAYER', 'HansPeterMayer']
'Mayor Oreja' => ['Jaime MAYOR OREJA', 'JaimeMayorOreja']
'Maňka' => ['Vladimír MAŇKA', 'VladimirManka']
'McAvan' => ['Linda McAVAN', 'LindaMcavan']
'McCarthy' => ['Arlene McCARTHY', 'ArleneMccarthy']
'McDonald' => ['Mary Lou McDONALD', 'MaryLouMcdonald']
'McGuinness' => ['Mairead McGUINNESS', 'MaireadMcguinness']
'Medina Ortega' => ['Manuel MEDINA ORTEGA', 'ManuelMedinaOrtega']
'Meijer' => ['Erik MEIJER', 'ErikMeijer']
'Menéndez del Valle' => ['Emilio MENÉNDEZ del VALLE', 'EmilioMenendezDelValle']
'Miguélez Ramos' => ['Rosa MIGUÉLEZ RAMOS', 'RosaMiguelezRamos']
'Mikko' => ['Marianne MIKKO', 'MarianneMikko']
'Mikolášik' => ['Miroslav MIKOLÁŠIK', 'MiroslavMikolasik']
'Millán Mon' => ['Francisco José MILLÁN MON', 'FranciscoJoseMillanMon']
'Mitchell' => ['Gay MITCHELL', 'GayMitchell']
'Mladenov' => ['Nickolay MLADENOV', 'NickolayMladenov']
'Mohácsi' => ['Viktória MOHÁCSI', 'ViktoriaMohacsi']
'Moraes' => ['Claude MORAES', 'ClaudeMoraes']
'Moreno Sánchez' => ['Javier MORENO SÁNCHEZ', 'JavierMorenoSanchez']
'Morgan' => ['Eluned MORGAN', 'ElunedMorgan']
'Morillon' => ['Philippe MORILLON', 'PhilippeMorillon']
'Morin' => ['Elisabeth MORIN', 'ElisabethMorin']
'Mote' => ['Ashley MOTE', 'AshleyMote']
'Musacchio' => ['Roberto MUSACCHIO', 'RobertoMusacchio']
'Muscat' => ['Joseph MUSCAT', 'JosephMuscat']
'Myller' => ['Riitta MYLLER', 'RiittaMyller']
'Mölzer' => ['Andreas MÖLZER', 'AndreasMolzer']
'Mănescu' => ['Ramona Nicole MĂNESCU', 'RamonaNicoleManescu']
'Naranjo Escobar' => ['Juan Andrés NARANJO ESCOBAR', 'JuanAndresNaranjoEscobar']
'Nassauer' => ['Hartmut NASSAUER', 'HartmutNassauer']
'Neris' => ['Catherine NERIS', 'CatherineNeris']
'Newton Dunn' => ['Bill NEWTON DUNN', 'BillNewtonDunn']
'Neyts-Uyttebroeck' => ['Annemie NEYTS-UYTTEBROECK', 'AnnemieNeytsUyttebroeck']
'Nicholson of Winterbourne' => ['Baroness NICHOLSON OF WINTERBOURNE', 'BaronessEmmaNicholsonOfWinterbourne']
'Niebler' => ['Angelika NIEBLER', 'AngelikaNiebler']
'Novak' => ['Ljudmila NOVAK', 'LjudmilaNovak']
'Obiols i Germà' => ['Raimon OBIOLS i GERMÀ', 'RaimonObiolsIGerma']
'Olajos' => ['Péter OLAJOS', 'PeterOlajos']
'Olbrycht' => ['Jan OLBRYCHT', 'JanOlbrycht']
'Onesta' => ['Gérard ONESTA', 'GerardOnesta']
'Onyszkiewicz' => ['Janusz ONYSZKIEWICZ', 'JanuszOnyszkiewicz']
'Oomen-Ruijten' => ['Ria OOMEN-RUIJTEN', 'RiaOomenRuijten']
'Ortuondo Larrea' => ['Josu ORTUONDO LARREA', 'JosuOrtuondoLarrea']
'Ouzký' => ['Miroslav OUZKÝ', 'MiroslavOuzky']
'Oviir' => ['Siiri OVIIR', 'SiiriOviir']
'Paasilinna' => ['Reino PAASILINNA', 'ReinoPaasilinna']
'Pack' => ['Doris PACK', 'DorisPack']
'Pahor' => ['Borut PAHOR', 'BorutPahor']
'Paleckis' => ['Justas Vincas PALECKIS', 'JustasVincasPaleckis']
'Panayotopoulos-Cassiotou' => ['Marie PANAYOTOPOULOS-CASSIOTOU', 'MariePanayotopoulosCassiotou']
'Panayotov' => ['Vladko Todorov PANAYOTOV', 'VladkoTodorovPanayotov']
'Pannella' => ['Marco PANNELLA', 'MarcoPannella']
'Panzeri' => ['Pier Antonio PANZERI', 'PierAntonioPanzeri']
'Papadimoulis' => ['Dimitrios PAPADIMOULIS', 'DimitriosPapadimoulis']
'Paparizov' => ['Atanas PAPARIZOV', 'AtanasPaparizov']
'Papastamkos' => ['Georgios PAPASTAMKOS', 'GeorgiosPapastamkos']
'Parish' => ['Neil PARISH', 'NeilParish']
'Paşcu' => ['Ioan Mircea PAŞCU', 'IoanMirceaPascu']
'Peillon' => ['Vincent PEILLON', 'VincentPeillon']
'Peterle' => ['Alojz PETERLE', 'AlojzPeterle']
'Pflüger' => ['Tobias PFLÜGER', 'TobiasPfluger']
'Piecyk' => ['Willi PIECYK', 'WilliPiecyk']
'Pieper' => ['Markus PIEPER', 'MarkusPieper']
'Pinheiro' => ['João de Deus PINHEIRO', 'JoaodeDeusPinheiro']
'Pinior' => ['Józef PINIOR', 'JozefPinior']
'Piotrowski' => ['Mirosław Mariusz PIOTROWSKI', 'MiroslawMariuszPiotrowski']
'Pirilli' => ['Umberto PIRILLI', 'UmbertoPirilli']
'Pirker' => ['Hubert PIRKER', 'HubertPirker']
'Piskorski' => ['Paweł Bartłomiej PISKORSKI', 'PawelBartlomiejPiskorski']
'Pleguezuelos Aguilar' => ['Francisca PLEGUEZUELOS AGUILAR', 'FranciscaPleguezuelosAguilar']
'Pleštinská' => ['Zita PLEŠTINSKÁ', 'ZitaPlestinska']
'Plumb' => ['Rovana PLUMB', 'RovanaPlumb']
'Podimata' => ['Anni PODIMATA', 'AnniPodimata']
'Podkański' => ['Zdzisław Zbigniew PODKAŃSKI', 'ZdzislawZbigniewPodkanski']
'Poettering' => ['Hans-Gert PÖTTERING', 'HansGertPottering']
'Pohjamo' => ['Samuli POHJAMO', 'SamuliPohjamo']
'Poignant' => ['Bernard POIGNANT', 'BernardPoignant']
'Polfer' => ['Lydie POLFER', 'LydiePolfer']
'Popa Mihaela' => ['Mihaela POPA', 'MihaelaPopa']
'Popa Nicolae Vlad' => ['Nicolae Vlad POPA', 'NicolaeVladPopa']
'Portas' => ['Miguel PORTAS', 'MiguelPortas']
'Posselt' => ['Bernd POSSELT', 'BerndPosselt']
'Prets' => ['Christa PRETS', 'ChristaPrets']
'Prodi' => ['Vittorio PRODI', 'VittorioProdi']
'Protasiewicz' => ['Jacek PROTASIEWICZ', 'JacekProtasiewicz']
'Purvis' => ['John PURVIS', 'JohnPurvis']
'Pęk' => ['Bogdan PĘK', 'BogdanPek']
'Pīks' => ['Rihards PĪKS', 'RihardsPiks']
'Rack' => ['Reinhard RACK', 'ReinhardRack']
'Radwan' => ['Alexander RADWAN', 'AlexanderRadwan']
'Raeva' => ['Bilyana Ilieva RAEVA', 'BilyanaIlievaRaeva']
'Ransdorf' => ['Miloslav RANSDORF', 'MiloslavRansdorf']
'Rapkay' => ['Bernhard RAPKAY', 'BernhardRapkay']
'Rasmussen' => ['Poul Nyrup RASMUSSEN', 'PoulNyrupRasmussen']
'Remek' => ['Vladimír REMEK', 'VladimirRemek']
'Reul' => ['Herbert REUL', 'HerbertReul']
'Ribeiro e Castro' => ['José RIBEIRO E CASTRO', 'JoseRibeiroECastro']
'Riera Madurell' => ['Teresa RIERA MADURELL', 'TeresaRieraMadurell']
'Ries' => ['Frédérique RIES', 'FrederiqueRies']
'Riis-Jørgensen' => ['Karin RIIS-JØRGENSEN', 'KarinRiisJorgensen']
'Rizzo' => ['Marco RIZZO', 'MarcoRizzo']
'Rogalski' => ['Bogusław ROGALSKI', 'BoguslawRogalski']
'Roithová' => ['Zuzana ROITHOVÁ', 'ZuzanaRoithova']
'Romagnoli' => ['Luca ROMAGNOLI', 'LucaRomagnoli']
'Romeva i Rueda' => ['Raül ROMEVA i RUEDA', 'RaulRomevaIRueda']
'Rosati' => ['Dariusz ROSATI', 'DariuszRosati']
'Roszkowski' => ['Wojciech ROSZKOWSKI', 'WojciechRoszkowski']
'Rothe' => ['Mechtild ROTHE', 'MechtildRothe']
'Roure' => ['Martine ROURE', 'MartineRoure']
'Rouček' => ['Libor ROUČEK', 'LiborRoucek']
'Rovsing' => ['Christian ROVSING', 'ChristianRovsing']
'Rutowicz' => ['Leopold Józef RUTOWICZ', 'LeopoldJozefRutowicz']
'Ryan' => ['Eoin RYAN', 'EoinRyan']
'Rübig' => ['Paul RÜBIG', 'PaulRubig']
'Sacconi' => ['Guido SACCONI', 'GuidoSacconi']
'Sakalas' => ['Aloyzas SAKALAS', 'AloyzasSakalas']
'Saks' => ['Katrin SAKS', 'KatrinSaks']
'Salafranca Sánchez-Neyra' => ['José Ignacio SALAFRANCA SÁNCHEZ-NEYRA', 'JoseIgnacioSalafrancaSanchezNeyra']
'Salinas García' => ['María Isabel SALINAS GARCÍA', 'MariaIsabelSalinasGarcia']
'Sanz Palacio' => ['Salvador Domingo SANZ PALACIO', 'SalvadorDomingoSanzPalacio']
'Saryusz-Wolski' => ['Jacek SARYUSZ-WOLSKI', 'JacekSaryuszWolski']
'Savary' => ['Gilles SAVARY', 'GillesSavary']
'Savi' => ['Toomas SAVI', 'ToomasSavi']
'Saïfi' => ['Tokia SAÏFI', 'TokiaSaifi']
'Schaldemose' => ['Christel SCHALDEMOSE', 'ChristelSchaldemose']
'Schapira' => ['Pierre SCHAPIRA', 'PierreSchapira']
'Schenardi' => ['Lydia SCHENARDI', 'LydiaSchenardi']
'Schierhuber' => ['Agnes SCHIERHUBER', 'AgnesSchierhuber']
'Schinas' => ['Margaritis SCHINAS', 'MargaritisSchinas']
'Schlyter' => ['Carl SCHLYTER', 'CarlSchlyter']
'Schmidt' => ['Frithjof SCHMIDT', 'FrithjofSchmidt']
'Schmidt Frithjof' => ['Frithjof SCHMIDT', 'FrithjofSchmidt']
'Schmidt Olle' => ['Olle SCHMIDT', 'OlleSchmidt']
'Schmitt' => ['Pál SCHMITT', 'PalSchmitt']
'Schnellhardt' => ['Horst SCHNELLHARDT', 'HorstSchnellhardt']
'Schroedter' => ['Elisabeth SCHROEDTER', 'ElisabethSchroedter']
'Schröder' => ['Jürgen SCHRÖDER', 'JurgenSchroder']
'Schulz' => ['Martin SCHULZ', 'MartinSchulz']
'Schuth' => ['Willem SCHUTH', 'WillemSchuth']
'Schwab' => ['Andreas SCHWAB', 'AndreasSchwab']
'Schöpflin' => ['György SCHÖPFLIN', 'GyorgySchopflin']
'Seeber' => ['Richard SEEBER', 'RichardSeeber']
'Segelström' => ['Inger SEGELSTRÖM', 'IngerSegelstrom']
'Seppänen' => ['Esko SEPPÄNEN', 'EskoSeppanen']
'Severin' => ['Adrian SEVERIN', 'AdrianSeverin']
'Siekierski' => ['Czesław Adam SIEKIERSKI', 'CzeslawAdamSiekierski']
'Silva Peneda' => ['José Albino SILVA PENEDA', 'JoseAlbinoSilvaPeneda']
'Simpson' => ['Brian SIMPSON', 'BrianSimpson']
'Siwiec' => ['Marek SIWIEC', 'MarekSiwiec']
'Skinner' => ['Peter SKINNER', 'PeterSkinner']
'Smith' => ['Alyn SMITH', 'AlynSmith']
'Sommer' => ['Renate SOMMER', 'RenateSommer']
'Sonik' => ['Bogusław SONIK', 'BoguslawSonik']
'Sornosa Martínez' => ['María SORNOSA MARTÍNEZ', 'MariaSornosaMartinez']
'Spautz' => ['Jean SPAUTZ', 'JeanSpautz']
'Speroni' => ['Francesco Enrico SPERONI', 'FrancescoEnricoSperoni']
'Staniszewska' => ['Grażyna STANISZEWSKA', 'GrazynaStaniszewska']
'Starkevičiūtė' => ['Margarita STARKEVIČIŪTĖ', 'MargaritaStarkeviciute']
'Stauner' => ['Gabriele STAUNER', 'GabrieleStauner']
'Stavreva' => ['Petya STAVREVA', 'PetyaStavreva']
'Sterckx' => ['Dirk STERCKX', 'DirkSterckx']
'Stevenson' => ['Struan STEVENSON', 'StruanStevenson']
'Stihler' => ['Catherine STIHLER', 'CatherineStihler']
'Stockmann' => ['Ulrich STOCKMANN', 'UlrichStockmann']
'Stoyanov' => ['Dimitar STOYANOV', 'DimitarStoyanov']
'Strejček' => ['Ivo STREJČEK', 'IvoStrejcek']
'Strož' => ['Daniel STROŽ', 'DanielStroz']
'Sturdy' => ['Robert STURDY', 'RobertSturdy']
'Sudre' => ['Margie SUDRE', 'MargieSudre']
'Sumberg' => ['David SUMBERG', 'DavidSumberg']
'Surján' => ['László SURJÁN', 'LaszloSurjan']
'Susta' => ['Gianluca SUSTA', 'GianlucaSusta']
'Svensson' => ['Eva-Britt SVENSSON', 'EvaBrittSvensson']
'Swoboda' => ['Hannes SWOBODA', 'HannesSwoboda']
'Szejna' => ['Andrzej Jan SZEJNA', 'AndrzejJanSzejna']
'Szent-Iványi' => ['István SZENT-IVÁNYI', 'IstvanSzentIvanyi']
'Szymański' => ['Konrad SZYMAŃSKI', 'KonradSzymanski']
'Szájer' => ['József SZÁJER', 'JozsefSzajer']
'Sánchez Presedo' => ['Antolín SÁNCHEZ PRESEDO', 'AntolinSanchezPresedo']
'Sógor' => ['Csaba SÓGOR', 'CsabaSogor']
'Søndergaard' => ['Søren Bo SØNDERGAARD', 'SorenBoSondergaard']
'Tannock' => ['Charles TANNOCK', 'CharlesTannock']
'Tarand' => ['Andres TARAND', 'AndresTarand']
'Thomsen' => ['Britta THOMSEN', 'BrittaThomsen']
'Thyssen' => ['Marianne THYSSEN', 'MarianneThyssen']
'Titford' => ['Jeffrey TITFORD', 'JeffreyTitford']
'Titley' => ['Gary TITLEY', 'GaryTitley']
'Tomaszewska' => ['Ewa TOMASZEWSKA', 'EwaTomaszewska']
'Tomczak' => ['Witold TOMCZAK', 'WitoldTomczak']
'Toubon' => ['Jacques TOUBON', 'JacquesToubon']
'Trakatellis' => ['Antonios TRAKATELLIS', 'AntoniosTrakatellis']
'Trautmann' => ['Catherine TRAUTMANN', 'CatherineTrautmann']
'Triantaphyllides' => ['Kyriacos TRIANTAPHYLLIDES', 'KyriacosTriantaphyllides']
'Trüpel' => ['Helga TRÜPEL', 'HelgaTrupel']
'Turmes' => ['Claude TURMES', 'ClaudeTurmes']
'Tzampazi' => ['Evangelia TZAMPAZI', 'EvangeliaTzampazi']
'Tőkés' => ['László TŐKÉS', 'LaszloTokes']
'Uca' => ['Feleknas UCA', 'FeleknasUca']
'Ulmer' => ['Thomas ULMER', 'ThomasUlmer']
'Urutchev' => ['Vladimir URUTCHEV', 'VladimirUrutchev']
'Vaidere' => ['Inese VAIDERE', 'IneseVaidere']
'Vakalis' => ['Nikolaos VAKALIS', 'NikolaosVakalis']
'Van Hecke' => ['Johan VAN HECKE', 'JohanVanHecke']
'Van Lancker' => ['Anne VAN LANCKER', 'AnneVanLancker']
'Van Orden' => ['Geoffrey VAN ORDEN', 'GeoffreyVanOrden']
'Vanhecke' => ['Frank VANHECKE', 'FrankVanhecke']
'Varela Suanzes-Carpegna' => ['Daniel VARELA SUANZES-CARPEGNA', 'DanielVarelaSuanzesCarpegna']
'Varvitsiotis' => ['Ioannis VARVITSIOTIS', 'IoannisVarvitsiotis']
'Vaugrenard' => ['Yannick VAUGRENARD', 'YannickVaugrenard']
'Ventre' => ['Riccardo VENTRE', 'RiccardoVentre']
'Veraldi' => ['Donato Tommaso VERALDI', 'DonatoTommasoVeraldi']
'Vergnaud' => ['Bernadette VERGNAUD', 'BernadetteVergnaud']
'Vidal-Quadras' => ['Alejo VIDAL-QUADRAS', 'AlejoVidalQuadras']
'Vigenin' => ['Kristian VIGENIN', 'KristianVigenin']
'Virrankoski' => ['Kyösti VIRRANKOSKI', 'KyostiVirrankoski']
'Visser' => ['Cornelis VISSER', 'CornelisVisser']
'Vlasto' => ['Dominique VLASTO', 'DominiqueVlasto']
'Vlasák' => ['Oldřich VLASÁK', 'OldrichVlasak']
'Voggenhuber' => ['Johannes VOGGENHUBER', 'JohannesVoggenhuber']
'Vălean' => ['Adina-Ioana VĂLEAN', 'AdinaIoanaValean']
'Wagenknecht' => ['Sahra WAGENKNECHT', 'SahraWagenknecht']
'Wallis' => ['Diana WALLIS', 'DianaWallis']
'Walter' => ['Ralf WALTER', 'RalfWalter']
'Weber Henri' => ['Henri WEBER', 'HenriWeber']
'Weber Manfred' => ['Manfred WEBER', 'ManfredWeber']
'Weber Renate' => ['Renate WEBER', 'RenateWeber']
'Weisgerber' => ['Anja WEISGERBER', 'AnjaWeisgerber']
'Westlund' => ['Åsa WESTLUND', 'AsaWestlund']
'Wieland' => ['Rainer WIELAND', 'RainerWieland']
'Wiersma' => ['Jan Marinus WIERSMA', 'JanMarinusWiersma']
'Wijkman' => ['Anders WIJKMAN', 'AndersWijkman']
'Willmott' => ['Glenis WILLMOTT', 'GlenisWillmott']
'Winkler' => ['Iuliu WINKLER', 'IuliuWinkler']
'Wise' => ['Thomas WISE', 'ThomasWise']
'Wohlin' => ['Lars WOHLIN', 'LarsWohlin']
'Wojciechowski Bernard' => ['Bernard WOJCIECHOWSKI', 'BernardWojciechowski']
'Wojciechowski Bernard Piotr' => ['Bernard WOJCIECHOWSKI', 'BernardWojciechowski']
'Wojciechowski Janusz' => ['Janusz WOJCIECHOWSKI', 'JanuszWojciechowski']
'Wortmann-Kool' => ['Corien WORTMANN-KOOL', 'CorienWortmannKool']
'Wurtz' => ['Francis WURTZ', 'FrancisWurtz']
'Yañez-Barnuevo García' => ['Luis YAÑEZ-BARNUEVO GARCÍA', 'LuisYanezBarnuevoGarcia']
'Zahradil' => ['Jan ZAHRADIL', 'JanZahradil']
'Zaleski' => ['Zbigniew ZALESKI', 'ZbigniewZaleski']
'Zani' => ['Mauro ZANI', 'MauroZani']
'Zapałowski' => ['Andrzej Tomasz ZAPAŁOWSKI', 'AndrzejTomaszZapalowski']
'Zappala\'' => ['Stefano ZAPPALA\'', 'StefanoZappala']
'Zappalà' => ['Stefano ZAPPALA\'', 'StefanoZappala']
'Zdravkova' => ['Dushana ZDRAVKOVA', 'DushanaZdravkova']
'Zieleniec' => ['Josef ZIELENIEC', 'JosefZieleniec']
'Zimmer' => ['Gabriele ZIMMER', 'GabrieleZimmer']
'Zlotea' => ['Marian ZLOTEA', 'MarianZlotea']
'Zwiefka' => ['Tadeusz ZWIEFKA', 'TadeuszZwiefka']
'Záborská' => ['Anna ZÁBORSKÁ', 'AnnaZaborska']
'de Brún' => ['Bairbre de BRÚN', 'BairbredeBrun']
'de Grandes Pascual' => ['Luis de GRANDES PASCUAL', 'LuisdeGrandesPascual']
'de Groen-Kouwenhoven' => ['Elly de GROEN-KOUWENHOVEN', 'EllydeGroenKouwenhoven']
'dos Santos' => ['Manuel António dos SANTOS', 'ManuelAntonioDosSantos']
'van Nistelrooij' => ['Lambert van NISTELROOIJ', 'LambertVanNistelrooij']
'van den Burg' => ['Ieke van den BURG', 'IekeVanDenBurg']
'von Wogau' => ['Karl von WOGAU', 'KarlVonWogau']
'Ó Neachtain' => ['Seán Ó NEACHTAIN', 'SeanONeachtain']
'Özdemir' => ['Cem ÖZDEMIR', 'CemOzdemir']
'Škottová' => ['Nina ŠKOTTOVÁ', 'NinaSkottova']
'Šťastný' => ['Peter ŠŤASTNÝ', 'PeterStastny']
'Ţicău' => ['Silvia-Adriana ŢICĂU', 'SilviaAdrianaTicau']
'Ždanoka' => ['Tatjana ŽDANOKA', 'TatjanaZdanoka']
'Železný' => ['Vladimír ŽELEZNÝ', 'VladimirZelezny']
'Resetarits' => ['Karin RESETARITS', 'KarinResetarits']
'Catania' => ['Giusto CATANIA', 'GiustoCatania']
'Markov' => ['Helmuth MARKOV', 'HelmuthMarkov']
'Maštálka' => ['Jiří MAŠTÁLKA', 'JiriMastalka']
'Meyer Pleite' => ['Willy MEYER PLEITE', 'WillyMeyerPleite']
'Morgantini' => ['Luisa MORGANTINI', 'LuisaMorgantini']
'Coûteaux' => ['Paul Marie COÛTEAUX', 'PaulMarieCouteaux']
'Sinnott' => ['Kathy SINNOTT', 'KathySinnott']
'Le Pen Marine' => ['Marine LE PEN', 'MarineLePen']
'Jørgensen' => ['Dan JØRGENSEN', 'DanJorgensen']
'Rocard' => ['Michel ROCARD', 'MichelRocard']
'Scheele' => ['Karin SCHEELE', 'KarinScheele']
'Sousa Pinto' => ['Sérgio SOUSA PINTO', 'SergioSousaPinto']
'Breyer' => ['Hiltrud BREYER', 'HiltrudBreyer']
'van Buitenen' => ['Paul van BUITENEN', 'PaulVanBuitenen']
'Rühle' => ['Heide RÜHLE', 'HeideRuhle']
'Staes' => ['Bart STAES', 'BartStaes']
'Cornillet' => ['Thierry CORNILLET', 'ThierryCornillet']
'Harkin' => ['Marian HARKIN', 'MarianHarkin']
'Hennis-Plasschaert' => ['Jeanine HENNIS-PLASSCHAERT', 'JeanineHennisPlasschaert']
'Klinz' => ['Wolf KLINZ', 'WolfKlinz']
'Lax' => ['Henrik LAX', 'HenrikLax']
'Czarnecki Marek Aleksander' => ['Marek Aleksander CZARNECKI', 'MarekAleksanderCzarnecki']
'Le Rachinel' => ['Fernand LE RACHINEL', 'FernandLeRachinel']
'Rivera' => ['Giovanni RIVERA', 'GiovanniRivera']
'Antoniozzi' => ['Alfredo ANTONIOZZI', 'AlfredoAntoniozzi']
'Bonsignore' => ['Vito BONSIGNORE', 'VitoBonsignore']
'Castiglione' => ['Giuseppe CASTIGLIONE', 'GiuseppeCastiglione']
'Deva' => ['Nirj DEVA', 'NirjDeva']
'Esteves' => ['Maria da Assunção ESTEVES', 'MariadaAssuncaoEsteves']
'Ferber' => ['Markus FERBER', 'MarkusFerber']
'Fernández Martín' => ['Fernando FERNÁNDEZ MARTÍN', 'FernandoFernandezMartin']
'Florenz' => ['Karl-Heinz FLORENZ', 'KarlHeinzFlorenz']
'Freitas' => ['Duarte FREITAS', 'DuarteFreitas']
'Garriga Polledo' => ['Salvador GARRIGA POLLEDO', 'SalvadorGarrigaPolledo']
'Gutiérrez-Cortines' => ['Cristina GUTIÉRREZ-CORTINES', 'CristinaGutierrezCortines']
'Itälä' => ['Ville ITÄLÄ', 'VilleItala']
'Iturgaiz Angulo' => ['Carlos José ITURGAIZ ANGULO', 'CarlosJoseIturgaizAngulo']
'Kaczmarek' => ['Filip KACZMAREK', 'FilipKaczmarek']
'Kamall' => ['Syed KAMALL', 'SyedKamall']
'McMillan-Scott' => ['Edward McMILLAN-SCOTT', 'EdwardMcmillanScott']
'Matsis' => ['Yiannakis MATSIS', 'YiannakisMatsis']
'Podestà' => ['Guido PODESTÀ', 'GuidoPodesta']
'Pomés Ruiz' => ['José Javier POMÉS RUIZ', 'JoseJavierPomesRuiz']
'Posdorf' => ['Horst POSDORF', 'HorstPosdorf']
'Queiró' => ['Luís QUEIRÓ', 'LuisQueiro']
'Quisthoudt-Rowohl' => ['Godelieve QUISTHOUDT-ROWOHL', 'GodelieveQuisthoudtRowohl']
'Sartori' => ['Amalia SARTORI', 'AmaliaSartori']
'Tajani' => ['Antonio TAJANI', 'AntonioTajani']
'Vatanen' => ['Ari VATANEN', 'AriVatanen']
'Veneto' => ['Armando VENETO', 'ArmandoVeneto']
'Vernola' => ['Marcello VERNOLA', 'MarcelloVernola']
'Zatloukal' => ['Tomáš ZATLOUKAL', 'TomasZatloukal']
'Zvěřina' => ['Jaroslav ZVĚŘINA', 'JaroslavZverina']
'Berlinguer' => ['Giovanni BERLINGUER', 'GiovanniBerlinguer']
'Kuhne' => ['Helmut KUHNE', 'HelmutKuhne']
'Lavarra' => ['Vincenzo LAVARRA', 'VincenzoLavarra']
'Pittella' => ['Gianni PITTELLA', 'GianniPittella']
'Roth-Behrendt' => ['Dagmar ROTH-BEHRENDT', 'DagmarRothBehrendt']
'Weiler' => ['Barbara WEILER', 'BarbaraWeiler']
'Crowley' => ['Brian CROWLEY', 'BrianCrowley']
'Didžiokas' => ['Gintaras DIDŽIOKAS', 'GintarasDidziokas']
'La Russa' => ['Romano Maria LA RUSSA', 'RomanoMariaLaRussa']
'Musumeci' => ['Sebastiano (Nello) MUSUMECI', 'SebastianoMusumeci']
'Tatarella' => ['Salvatore TATARELLA', 'SalvatoreTatarella']
'Toia' => ['Patrizia TOIA', 'PatriziaToia']
'Batten' => ['Gerard BATTEN', 'GerardBatten']
'Bloom' => ['Godfrey BLOOM', 'GodfreyBloom']
'Badia I Cutchet' => ['Maria BADIA i CUTCHET', 'MariaBadiaICutchet']
'Désir' => ['Harlem DÉSIR', 'HarlemDesir']
'Dührkop Dührkop' => ['Bárbara DÜHRKOP DÜHRKOP', 'BarbaraDuhrkopDuhrkop']
'Fava' => ['Claudio FAVA', 'ClaudioFava']
'Gill' => ['Neena GILL', 'NeenaGill']
'Harangozó' => ['Gábor HARANGOZÓ', 'GaborHarangozo']
'Howitt' => ['Richard HOWITT', 'RichardHowitt']
'Matsouka' => ['Maria MATSOUKA', 'MariaMatsouka']
'Napoletano' => ['Pasqualina NAPOLETANO', 'PasqualinaNapoletano']
'Öger' => ['Vural ÖGER', 'VuralOger']
'Tabajdi' => ['Csaba Sándor TABAJDI', 'CsabaSandorTabajdi']
'Cashman' => ['Michael CASHMAN', 'MichaelCashman']
'Mathieu' => ['Véronique MATHIEU', 'VeroniqueMathieu']
'Muscardini' => ['Cristiana MUSCARDINI', 'CristianaMuscardini']
'Buitenweg' => ['Kathalijne Maria BUITENWEG', 'KathalijneMariaBuitenweg']
'Costa Paolo' => ['Paolo COSTA', 'PaoloCosta']
'Mulder' => ['Jan MULDER', 'JanMulder']
'Watson' => ['Graham WATSON', 'GrahamWatson']
'Evans Jonathan' => ['Jonathan EVANS', 'JonathanEvans']
'Gargani' => ['Giuseppe GARGANI', 'GiuseppeGargani']
'Gawronski' => ['Jas GAWRONSKI', 'JasGawronski']
'Marini' => ['Catiuscia MARINI', 'CatiusciaMarini']
'Mayer Hans-Peter' => ['Hans-Peter MAYER', 'HansPeterMayer']
'Méndez de Vigo' => ['Íñigo MÉNDEZ DE VIGO', 'InigoMendezdeVigo']
'Nicholson' => ['James NICHOLSON', 'JamesNicholson']
'Patrie' => ['Béatrice PATRIE', 'BeatricePatrie']
'Schröder Jürgen' => ['Jürgen SCHRÖDER', 'JurgenSchroder']
'Angelilli' => ['Roberta ANGELILLI', 'RobertaAngelilli']
'in \'t Veld' => ['Sophia in \'t VELD', 'SophiaInTVeld']
'Őry' => ['Csaba ŐRY', 'CsabaOry']
