#!/usr/bin/perl

# Copyright 2007 Gérald Sédrati-Dinet
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files are located under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage : $0 [-h] [-p <path_to_db_env>] [<container> [<NameOfPolitician>]]
where <container> can be candidates, mps, meps or ministers
(for convenience the final "s" can be omitted, also ".dbxml" can be
appended), if this optional parameter is set, the ouput result will
show all politicians who have registered opinions in this container;

and <NameOfPolitician> is the wiki name of a politician, if this
optional parameter is set, the <container> parameter should also be set,
and the output result will simply show whether this politian has some
opinions registered;

if both optional parameters are omitted, the output result will
show all politicians who have registered opinions in all containers.

Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

my $container = shift || '';
my $wiki_name = shift || '';
$container =~ s/s?(\d*)(?:\.dbxml)?$/s$1/ if $container;

my @containers;
if ($container) {
  push @containers, $container;
}
else {
  @containers = (
    'meps',
    'en_meps',
    'mps',
  );
}

foreach my $theContainer (@containers) {
  print "\nFor container: <$theContainer>:\n";
  eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv, Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);
    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query;
    if ($wiki_name) {
      $query = <<"QUERY";
        for \$politician in collection("$theContainer.dbxml")/politician[infos/name/wiki='$wiki_name' and opinions]
        let \$name := \$politician/infos/name
        order by upper-case(\$name/last/string()),  upper-case(\$name/first/string())
        return
          <result>
            {\$name/wiki}
            {\$politician/opinions}
          </result>
QUERY
  }
    else {
      $query = <<"QUERY";
        for \$politician in collection("$theContainer.dbxml")/politician[opinions]
        let \$name := \$politician/infos/name
        order by upper-case(\$name/last/string()),  upper-case(\$name/first/string())
        return
          <result>
            {\$name/wiki}
            {\$politician/opinions}
          </result>
QUERY
    }
    
    #  Perform the query
    eval {
      my $results = $theMgr->query($query);
      my $value;
      while( $results->next($value) ) {
        my $xml_opinions = XML::Twig->new(output_encoding => 'ISO-8859-1')->parse($value);
        my $name = $xml_opinions->root->first_child_text('wiki');
        print "$name\n";
      }
    };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
  };

  if (my $e = catch std::exception) {
    warn "Exception:\n";
    warn $e->what() . "\n";
    exit( -1 );
  }
  elsif ($@) {
    warn "Query failed\n";
    warn $@;
    exit( -1 );
  }
}
