#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;
use utf8;
use open ':utf8';

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';
my $output_dir = "$root_dir/mediawiki/meps";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <MemberOfEuropeanParliamentWikiName>
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my $mp_wiki = shift or die "You should provide the wiki name of the Member of the Parliament, see $0 -h\n";

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
          Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");

# Get the document
eval {
  my $theDocument = $container->getDocument($mp_wiki);
  my $docString = $theDocument->getContent();

  # Build xml tree
  my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($docString);

  # Extract infos
  my $current_mep = (defined $xml->root->first_child('container')->att('current') and $xml->root->first_child('container')->att('current') eq 'true') ? 1 : 0;
  my $infos = $xml->root->first_child('infos');
  my $name = $infos->first_child('name');
  my $full_name = $name->first_child_text('full');
  my $first_name = $name->first_child_text('first');
  my $last_name = $name->first_child_text('last');
  my $wiki_name = $name->first_child_text('wiki');
  my $pic = $infos->first_child_text('picture');
  my $birth = $infos->first_child('birth');
  my $role = ($current_mep ? '' : 'ex-').'eurodéputé'.(($birth->first_child_text('gender') eq 'Né')?'':'e');
  my $birth_date = $birth->first_child('date');
  my $birth_place = $birth->first_child('place');
  my $birth_str = sprintf "''%s le'' %02d %s %04d ''à'' %s", $birth->first_child_text('gender'), $birth_date->first_child_text('day'), $birth_date->first_child_text('month'), $birth_date->first_child_text('year'), $birth_place->first_child_text('city');
  my $constituency = $infos->first_child('constituency');
  my $country = $constituency->first_child('country')->first_child_text('name');
  my $country_code = $constituency->first_child('country')->first_child_text('code');
  my $grp = $infos->first_child('group');
  my $grp_str = $grp->first_child_text('name');
  my $party = $grp->first_child_text('party');
  my $grp_abbrev = $grp->first_child_text('abbreviation');
  (my $grp_wiki = $grp_abbrev) =~ s/[\/-].+$//o;

  my $contact = $xml->root->first_child('contact');
  my $address = "* " . join("\n* ", map({($_->first_child('label')?$_->first_child_text('label')."<br />":'').($_->first_child('building')?'[[Eurodeputes_'.(($_->first_child('building')->first_child_text('abbreviation') eq '-')?(($_->first_child_text('city') eq 'Bruxelles')?'BXL_SDF':'STR_SDF'):$_->first_child('building')->first_child_text('abbreviation').'_'.(($_->first_child_text('city') eq 'Bruxelles')?substr($_->first_child_text('office'),0,2):substr($_->first_child_text('office'),1,2))).'|'.(($_->first_child('building')->first_child_text('abbreviation') eq '-')?'':'Bâtiment ').$_->first_child('building')->first_child_text('name').(($_->first_child('building')->first_child_text('abbreviation') eq '-')?', bureau non attribué':' '.$_->first_child_text('office')).']]<br />':'').($_->first_child('street')?$_->first_child_text('street').',':'').($_->first_child('postcode')?' '.$_->first_child_text('postcode'):'').($_->first_child('city')?' '.$_->first_child_text('city'):'').($_->first_child('district')?', '.$_->first_child_text('district'):'').($_->first_child('phone')?(" <br />''Tél.'' : ".join('/', (map {(my $phone_url = $_) =~ s/ //g;"[callto://$phone_url $_]"} $_->children_text('phone')))):'').($_->first_child('fax')?("<br />''Fax'' : ".join('/', $_->children_text('fax'))):'')} $contact->children('address')))."\n";

  # Hack for telephone/fax without address
  $address =~ s/^\*  - /* /om;

  (my $icon_phone) = grep /^\+32 2 28 45/, map {$_->text} $contact->descendants('phone');
  if ($icon_phone) {
    (my $url_phone = $icon_phone) =~ s/ //g;
    $icon_phone = "{{#icon:Cliquez_pour_appeler.png|$icon_phone||callto://$url_phone}} ";
  }
  else {
    $icon_phone = '';
  }

  my @mails = $contact->children_text('email');
  my @webs = map {
    (my $web_str = $_->text) =~ s#^http://##o;
    my $type=$_->att('type')?ucfirst($_->att('type')):'Site web';
    [$_->text(), $web_str, $type];
  } $contact->children('web');

  my @functions;
  foreach my $func ($xml->root->first_child('functions')->children('function')) {
    my $label = $func->first_child_text('label');
    my $role = $func->first_child_text('role');
    my $abbrev = $func->first_child_text('abbreviation');
    if ($abbrev) {
      $label = "[[Eurodeputes_$abbrev|$label]]";
    }
    elsif ($label eq 'Conférence des présidents') {
      $label = "[[Eurodeputes_presidents_groupes|$label]]";
    }
    my $func_str = "$label ($role)";
    my $begin_term = $func->first_child('begin_term');
    my $end_term = $func->first_child('end_term');
    if ($begin_term and $end_term) {
      my $begin_day = $begin_term->first_child_text('day');
      my $begin_month = $begin_term->first_child_text('month');
      my $begin_year = $begin_term->first_child_text('year');
      my $end_day = $end_term->first_child_text('day');
      my $end_month = $end_term->first_child_text('month');
      my $end_year = $end_term->first_child_text('year');
      $func_str = "$begin_day/$begin_month/$begin_year - $end_day/$end_month/$end_year : $func_str";
    }
    push @functions, $func_str;
  }

  my @positions = map {$_->text} $xml->root->first_child('cv')->children('position');

  my $activities = $xml->root->first_child('activities');
  my $questions_url = $activities->first_child_text('questions');
  my $motions_url = $activities->first_child_text('motions');
  my $reports_url = $activities->first_child_text('reports');
  my $opinions_url = $activities->first_child_text('opinions');
  my $debates_url = $activities->first_child_text('debates');
  my $declarations_url = $activities->first_child_text('declarations');

  my @scores=();
  if ($xml->root->first_child('scores')) {
    @scores = map {"* ".$_->first_child_text('date')." - [http://www.laquadrature.net/wiki/".$_->first_child_text('wiki')."_details_par_note?showmep=$wiki_name ".$_->first_child_text('label')."] '''".$_->first_child_text('value')."/100'''"} $xml->root->first_child('scores')->children('score');
  }

  my @opinions=();
  if ($xml->root->first_child('opinions')) {
    @opinions = map {"* ".$_->first_child_text('date')." [".$_->first_child_text('url')." ".$_->first_child_text('title')."]\n\n".$_->first_child_text('content')."\n"} $xml->root->first_child('opinions')->children('opinion');
  }

  # Build mediawiki page
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[$wiki_name|English]]''' - '''[[$wiki_name/fr|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : $full_name, $role==

<div style="float:right">{{#icon:$pic|$full_name}}</div>

===Informations générales===

* $birth_str
* ''Pays'' : {{#icon:$country_code.png|$country||Eurodeputes_$country_code}} [[Eurodeputes_$country_code|$country]]
* ''Groupe politique'' : {{#icon:$grp_wiki.png|$grp_abbrev||Eurodeputes_$grp_wiki}} [[Eurodeputes_$grp_wiki|$grp_str ($grp_abbrev)]]<br/>''Parti'' : $party
MEDIAWIKI_PAGE

  if ($current_mep) {
    $mediawiki_page .= << "MEDIAWIKI_PAGE";
{{Robelbox|theme=13|title=Contact|icon=Calligraphy.png|iconwidth=40px|width=60%}}
<div style="{{Robelbox/pad}}">
<div style="float:right">$icon_phone</div>
$address
MEDIAWIKI_PAGE
  
    foreach my $mail (@mails) {
      my $mail_str;
      (($mail_str = $mail) =~ s/@/(à)/o);
      $mediawiki_page .= "* ''Courriel'' : [mailto:$mail $mail_str]\n" if $mail;
    }
    foreach my $web (@webs) {
      $mediawiki_page .= "* ''$web->[2]'' : [$web->[0] $web->[1]]\n";
    }

    $mediawiki_page .= << "MEDIAWIKI_PAGE";
</div>
{{Robelbox/close}}

MEDIAWIKI_PAGE
  }

  if (scalar @functions) {
    $mediawiki_page .= "\n====Fonctions au Parlement européen====\n\n";
    foreach my $function (@functions) {
      $mediawiki_page .= "* $function\n";
    }
  }

  if (scalar @positions) {
    $mediawiki_page .= "\n====Curriculum Vitae====\n\n";
    foreach my $pos (@positions) {
      $mediawiki_page .= "* $pos\n";
    }
  }

  $mediawiki_page .= <<"MEDIAWIKI";

----

===Votes===

MEDIAWIKI

  foreach my $score (@scores) {
    $mediawiki_page .= "$score\n";
  }

  $mediawiki_page .= <<"MEDIAWIKI";

----

===Prises de positions===

====Sources d'informations====

* [$questions_url Questions posées au Parlement européen]
* [$motions_url Propositions de résolutions au Parlement européen]
* [$reports_url Rapports au Parlement européen]
* [$opinions_url Avis au Parlement européen]
* [$debates_url Interventions en séance plénière au Parlement européen]
* [$declarations_url Déclarations écrites au Parlement européen]
* [http://fr.wikipedia.org/wiki/${first_name}_${last_name} Wikipédia]
* [http://www.google.fr/search?q=$first_name+$last_name Google]

====Positions====

''Merci d'enrichir cette partie en y rapportant les prises de positions de $full_name concernant les sujets traités par La Quadrature du Net (consultez la page [[Help:Memoire_politique]] pour savoir comment faire).''

MEDIAWIKI

  foreach my $opinion (@opinions) {
    $opinion =~ s/&lt;/</go;
    $mediawiki_page .= "$opinion\n";
  }
  $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";


  # ISO-8859-1 doesn't include some characters
  #$mediawiki_page =~ s/\x{153}/&oelig;/g;
  #$mediawiki_page =~ s/\x{2019}/'/g;

  # Output mediawiki page
  my $mediawiki_location = "$output_dir/$wiki_name.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

};

if (my $e = catch std::exception) {
  warn "getDocument failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "getDocument failed\n";
  warn $@;
  exit( -1 );
}
