#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_groups";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_groups
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my %groups_order = (
  'PPE-DE'    => 1,
  'PSE'       => 2,
  'ALDE'      => 3,
  'UEN'       => 4,
  'Verts/ALE' => 5,
  'GUE/NGL'   => 6,
  'IND/DEM'   => 7,
  'NI'        => 8,
);

my %group_meps;
my %group_prez;

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
for \$group in distinct-values(collection("dbxml:/$theContainer.dbxml")/politician/infos/group/abbreviation)
return
  <result>
    <code>{\$group}</code>
    {
    for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true" and infos/group/abbreviation/string()=\$group]
    order by upper-case(\$deputy/infos/name/last/string()), upper-case(\$deputy/infos/name/first/string())
    return
      <deputy>
        <group>{ \$deputy/infos/group/name/string() }</group>
        <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "/fr|", \$deputy/infos/name/full/string(), "]]") }</name>
        <country>{ concat("{{#icon:", \$deputy/infos/constituency/country/code/string(), ".png|", \$deputy/infos/constituency/country/name/string(), "||Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "}} [[Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "|", \$deputy/infos/constituency/country/name/string(), "]]") }</country>
        <country_name>{ \$deputy/infos/constituency/country/name/string() }</country_name>
        <country_code>{ \$deputy/infos/constituency/country/code/string() }</country_code>
        <role>{ \$deputy/infos/group/\@role }</role>
        <party>{ \$deputy/infos/group/party/string() }</party>
        <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tél : [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
        <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax : ", \$fax/string()), "<br />") }</fax>
        <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", "(à)"), "]"), "<br />") }</email>
        <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
        <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
      </deputy>
    }
  </result>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $code =  $xml->root->first_child_text('code');
          foreach my $mp ($xml->root->children('deputy')) {
            $group_meps{$code}->{NAME} = $mp->first_child_text('group');
            my %mp_data;
            $mp_data{NAME} = $mp->first_child_text('name');
            $mp_data{COUNTRY} = $mp->first_child_text('country');
            $mp_data{PARTY} = $mp->first_child_text('party');
            $mp_data{TEL} = $mp->first_child_text('tel');
            $mp_data{TEL} =~ s/Tél : \[callto:\/\/\+32/Tél Bxl. : [callto:\/\/+32/go;
            $mp_data{TEL} =~ s/Tél : \[callto:\/\/\+33/Tél Str. : [callto:\/\/+33/go;
            $mp_data{FAX} = $mp->first_child_text('fax');
            $mp_data{FAX} =~ s/Fax : \+32/Fax Bxl. : +32/go;
            $mp_data{FAX} =~ s/Fax : \+33/Fax Str. : +33/go;
            $mp_data{EMAIL} = $mp->first_child_text('email');
            my $bxl_addr = $mp->first_child('bxl')->first_child('address');
            $mp_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
            if ($mp_data{BXL} eq '- En cours de déménagement') {
              $mp_data{BXL} = '[[Eurodeputes_BXL_SDF|-]]';
            }
            else {
              $mp_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
            }

            my $str_addr = $mp->first_child('str')->first_child('address');
            $mp_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
            if ($mp_data{STR} eq '- En cours de déménagement') {
              $mp_data{STR} = '[[Eurodeputes_STR_SDF|-]]';
            }
            else {
              $mp_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
            }

            my $role = $mp->first_child('role')->att('role');
            if ($role ne 'Membre') {
              $mp_data{NAME} =~ s/]]$/ ($role)]]/o;
            }

            push @{$group_meps{$code}->{MEPS}}, \%mp_data;
          }
        }

  # Query for presidents
      $query = <<"QUERY";
for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true" and functions/function/label/string()="Conférence des présidents"]
order by upper-case(\$deputy/infos/name/last/string()), upper-case(\$deputy/infos/name/first/string())
return
  <deputy>
    <role>{ \$deputy/functions/function[label/string()="Conférence des présidents"]/role/string() }</role>
    <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "/fr|", \$deputy/infos/name/full/string(), "]]") }</name>
    <country>{ concat("{{#icon:", \$deputy/infos/constituency/country/code/string(), ".png|", \$deputy/infos/constituency/country/name/string(), "||Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "}} [[Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "|", \$deputy/infos/constituency/country/name/string(), "]]") }</country>
    <country_name>{ \$deputy/infos/constituency/country/name/string() }</country_name>
    <country_code>{ \$deputy/infos/constituency/country/code/string() }</country_code>
    <code>{ \$deputy/infos/group/abbreviation/string() }</code>
    <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tél : [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
    <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax: ", \$fax/string()), "<br />") }</fax>
    <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "\@", "(à)"), "]"), "<br />") }</email>
    <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
    <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
  </deputy>
QUERY
      # Perform the query for presidents
        my $results_prez = $theMgr->query($query);
        my $value_prez;
        while( $results_prez->next($value_prez) ) {
          my $xml_prez = XML::Twig->new(output_encoding => 'UTF-8')->parse($value_prez);
          my $prez = $xml_prez->root;
          my %prez_data;
          $prez_data{NAME} = $prez->first_child_text('name');
          $prez_data{COUNTRY} = $prez->first_child_text('country');
          my $prez_grp = $prez->first_child_text('code');
          $prez_data{TEL} = $prez->first_child_text('tel');
          $prez_data{TEL} =~ s/Tél : \[callto:\/\/\+32/Tél Bxl.: [callto:\/\/+32/go;
          $prez_data{TEL} =~ s/Tél : \[callto:\/\/\+33/Tél Str.: [callto:\/\/+33/go;
          $prez_data{FAX} = $prez->first_child_text('fax');
          $prez_data{FAX} =~ s/Fax : \+32/Fax Bxl. : +32/go;
          $prez_data{FAX} =~ s/Fax : \+33/Fax Str. : +33/go;
          $prez_data{EMAIL} = $prez->first_child_text('email');
          my $bxl_addr = $prez->first_child('bxl')->first_child('address');
          $prez_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
          if ($prez_data{BXL} eq '- En cours de déménagement') {
            $prez_data{BXL} = '[[Eurodeputes_BXL_SDF|-]]';
          }
          else {
            $prez_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
          }

          my $str_addr = $prez->first_child('str')->first_child('address');
          $prez_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
          if ($prez_data{STR} eq '- En cours de déménagement') {
            $prez_data{STR} = '[[Eurodeputes_STR_SDF|-]]';
          }
          else {
            $prez_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
          }

          my $role = $prez->first_child_text('role');
          if ($role ne 'Membre') {
            $prez_data{NAME} =~ s/]]$/ ($role)]]/o;
            # Hack to have president of group of president first_listed
            $prez_grp = "PREZ_$prez_grp";
            $groups_order{$prez_grp} = 0;
          }

          push @{$group_prez{$prez_grp}}, \%prez_data;
        }
      };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page for navigation
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_by_group|English]]''' - '''[[Eurodeputes_par_groupe|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : coordonnées des eurodéputés par groupe politique==

{|- style="background:none;"
MEDIAWIKI_PAGE

my $nb_col = 1;
my $col = 0;
foreach my $group_code (sort { $groups_order{$a} <=> $groups_order{$b} } keys %group_meps) {
  my $group_wiki = wikify_grp($group_code);
  $mediawiki_page .= "| {{#icon:$group_wiki.png|$group_meps{$group_code}->{NAME}||Eurodeputes_$group_wiki}}\n";
  $mediawiki_page .= "| [[Eurodeputes_$group_wiki|$group_meps{$group_code}->{NAME}]]\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";

$mediawiki_page .= "\n\n----\n\n[[Eurodeputes_presidents_groupes|Membres de la Conférence des présidents]]\n";
$mediawiki_page .= "\n\n----\n\nRetour à la page d'accueil de [[Memoire_politique|Mémoire politique]]\n";
$mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page
my $mediawiki_location = "$output_dir/Eurodeputes_par_groupe.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki page for presidents
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_groups_presidents|English]]''' - '''[[Eurodeputes_presidents_groupes|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : coordonnées des eurodéputés membres de la Conférence des présidents==

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Groupe !! Eurodéputé(e) !! Pays !! Contact !! Bureaux\n";
$mediawiki_page .= "|-\n";
foreach my $group_code (sort { $groups_order{$a} <=> $groups_order{$b} } keys %group_prez) {
  foreach my $deputy (@{$group_prez{$group_code}}) {
    $group_code =~ s/^PREZ_//o;
    my $group_wiki = wikify_grp($group_code);
    $mediawiki_page .= "| {{#icon:$group_wiki.png|$group_code||Eurodeputes_$group_wiki}} [[Eurodeputes_$group_wiki|$group_code]] || $deputy->{NAME} || $deputy->{COUNTRY} || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL} || Bruxelles : $deputy->{BXL}<br />$deputy->{STR}\n";
    $mediawiki_page .= "|-\n";
  }
}
$mediawiki_page =~ s/\|-\n$/|}\n/so;
$mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_groupe|Groupes politiques]]\n";
$mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page
$mediawiki_location = "$output_dir/Eurodeputes_presidents_groupes.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";


# Build mediawiki pages for each group
foreach my $group_code (sort { $groups_order{$a} <=> $groups_order{$b} } keys %group_meps) {
  my $group_wiki = wikify_grp($group_code);
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
  __NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_$group_wiki|English]]''' - '''[[Eurodeputes_$group_wiki|Français]]'''</td></tr></table></div>

==[[Memoire_politique|Mémoire politique]] : {{#icon:$group_wiki.png|$group_meps{$group_code}->{NAME}}} $group_meps{$group_code}->{NAME}, coordonnées des eurodéputés==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
  $mediawiki_page .= "! Eurodéputé(e) !! Pays !! Parti !! Contact !! Bureaux\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy (@{$group_meps{$group_code}->{MEPS}}) {
    $mediawiki_page .= "| $deputy->{NAME} || $deputy->{COUNTRY} || $deputy->{PARTY} || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL} || Bruxelles : $deputy->{BXL}<br />Strasbourg : $deputy->{STR}\n";
    $mediawiki_page .= "|-\n";
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n-----\n\n[[Eurodeputes_par_groupe|Autres groupes politiques]]\n";
  $mediawiki_page .= "[[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

  # Output mediawiki page
  my $mediawiki_location = "$output_dir/Eurodeputes_$group_wiki.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  $mediawiki_page =~ s/\x{2019}/'/g;
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

sub wikify_grp {
  my $str = shift;
  $str =~ s/[\/-].+$//o;
  return $str;
}
