#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps.dbxml';
my $output_dir = "$root_dir/mediawiki/mps_names";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically mps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: mps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/mps_names
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(\d*)(?:\.dbxml)?$/s$1/;
my $legislature = $1;

my @mps;
my @outgoing_mps;

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
      for \$deputy in collection("$theContainer.dbxml")/politician
      let \$last := upper-case(\$deputy/infos/name/last/string()),
        \$first := upper-case(\$deputy/infos/name/first/string())
      order by \$last, \$first
      return
        <result>
          <current>{ \$deputy/container/\@current/string() }</current>
          <last>{ \$last }</last>
          <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "|", \$deputy/infos/name/first/string(), " ", \$deputy/infos/name/last/string(), "]]") }</name>
          <group>{ \$deputy/infos/group/name/string() }</group>
          <party>{ \$deputy/infos/group/abbreviation/string() }</party>
          <department number="{\$deputy/infos/constituency/department/number/string()}">{ \$deputy/infos/constituency/department/name/string() }</department>
          <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tél : [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
          <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax : ", \$fax/string()), "<br />") }</fax>
          <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", "(à)"), "]"), "<br />") }</email>
        </result>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my %mp_data;
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          $mp_data{NAME} = $xml->root->first_child_text('name');
          $mp_data{GROUP} = $xml->root->first_child_text('group');
          $mp_data{PARTY} = $xml->root->first_child_text('party');
          $mp_data{DEPARTMENT} = $xml->root->first_child_text('department');
          $mp_data{DEP} = $xml->root->first_child('department')->att('number');
          ($mp_data{TEL} = $xml->root->first_child_text('tel')) =~ s/callto:\/\/0/callto:\/\/+33/g;
          $mp_data{FAX} = $xml->root->first_child_text('fax');
          $mp_data{EMAIL} = $xml->root->first_child_text('email');
          my $last = $xml->root->first_child_text('last');
          my $initial = substr $last, 0, 1;
          if ($xml->root->first_child_text('current') eq 'true') {
            push @mps, \%mp_data;
          }
          else {
            push @outgoing_mps, \%mp_data;
          }
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page for mps
my $mediawiki_page = "__NOEDITSECTION__\n==[[Memoire_politique|Mémoire politique]] : coordonnées des députés par ordre alphabétique==\n\n";
$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
$mediawiki_page .= "! Député(e) !! Parti !! Département !! Tél/Fax !! Courriel\n";
$mediawiki_page .= "|-\n";

foreach my $mp (@mps) {
  my $party_wiki;
  if ($mp->{GROUP} eq "Députés n'appartenant à aucun groupe") {
    $party_wiki = 'NI'
  }
  else {
    ($party_wiki = $mp->{PARTY}) =~ s/^Ap\. //o;
    $party_wiki = uc($party_wiki);
  }
  $mediawiki_page .= "| $mp->{NAME} || [[Deputes".($legislature?"_$legislature":'')."_$party_wiki|$mp->{PARTY}]] || [[Deputes".($legislature?"_$legislature":'')."_$mp->{DEP}|$mp->{DEPARTMENT}]] || $mp->{TEL}<br />$mp->{FAX} || $mp->{EMAIL}\n";
  $mediawiki_page .= "|-\n";
}

$mediawiki_page =~ s/\|-\n$/|}\n/so;
$mediawiki_page .= "\n----\n\nRetour à la page d'accueil de [[Memoire_politique|Mémoire politique]]\n";
$mediawiki_page .= "[[Category:Députés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page
my $mediawiki_location = "$output_dir/Deputes".($legislature?"_$legislature":'')."_par_nom.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki page for outgoing mps
$mediawiki_page = "__NOEDITSECTION__\n==[[Memoire_politique|Mémoire politique]] : députés dont le mandat est clos==\n\n";
$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
$mediawiki_page .= "! Député(e) !! Parti !! Département\n";
$mediawiki_page .= "|-\n";

foreach my $mp (@outgoing_mps) {
  my $party_wiki;
  if ($mp->{GROUP} eq "Députés n'appartenant à aucun groupe") {
    $party_wiki = 'NI'
  }
  else {
    ($party_wiki = $mp->{PARTY}) =~ s/^Ap\. //o;
    $party_wiki = uc($party_wiki);
  }
  $mediawiki_page .= "| $mp->{NAME} || [[Deputes_$party_wiki|$mp->{PARTY}]] || [[Deputes_$mp->{DEP}|$mp->{DEPARTMENT}]]\n";
  $mediawiki_page .= "|-\n";
}

$mediawiki_page =~ s/\|-\n$/|}\n/so;
$mediawiki_page .= "\n----\n\nRetour à la page d'accueil de [[Memoire_politique|Mémoire politique]]\n";
$mediawiki_page .= "[[Category:Députés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page
$mediawiki_location = "$output_dir/Deputes_sortants.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";
