#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use encoding 'utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use LWP::UserAgent;
use Getopt::Long;
use Date::Manip;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
    'path2dbenv|p=s' => \$path2DbEnv,
    'help|h'         =>
        sub {
            print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] <MepWikiName>
where <MepWikiName> is the wiki name of the mep
Options can be:
  --container, -c <container>:       DB XML container for MEPs
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --help, -h:                        print this message
USAGE
            exit 0;
        }
);

my $wiki_name = shift or die "You should provide the wiki name of the politician, see $0 -h\n";
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    warn "Opening $path2DbEnv/$theContainer.dbxml\n";
    $env->open($path2DbEnv, Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");
    my $updateContext = $theMgr->createUpdateContext();

    # Get the document
    my $theDocument = $container->getDocument($wiki_name);
    my $docString = $theDocument->getContent();
    my $docName = $theDocument->getName();

    # Build xml tree
    my $xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($docString);

    # Set current attribute
    $xml->root->first_child('container')->set_att('current' => 'false');

    # Set new url on europarl website
    my $old_url =  $xml->root->first_child('contact')->first_child('web[@type="Page sur le site du Parlement européen"]');
    my $id_mep =  $old_url->text;
    $id_mep =~ s/^.+id=//o;
    my $outgoing_url = "http://www.europarl.europa.eu/members/public/inOut/viewOutgoing.do?language=FR&id=$id_mep";
    $old_url->set_att('type', 'Ancienne page sur le site du Parlement européen');
    $old_url->insert_new_elt('before', 'web', {'type'=>'Page sur le site du Parlement européen'}, $outgoing_url);

    # Try to get more functions from europarl website
    # Get page
    my $ua = LWP::UserAgent->new;
    $ua->agent("$0 (gibus perl script to fetch information for http://www.laquadrature.net/wiki/)");
    $ua->env_proxy;
    my $headers = HTTP::Headers->new('Accept-Language' => 'fr');
    my $outgoing_req = HTTP::Request->new(GET => $outgoing_url, $headers);
    warn "Fetching $outgoing_url\n";
    my $outgoing_res = $ua->request($outgoing_req);

    if ($outgoing_res->is_success) {
        # Extract functions
        my $outgoing_html = $outgoing_res->content;
        my ($functions_txt) = ($outgoing_html =~ /<!-- END : MEP ID CARD -->\s*<br\/>\s*<table[^>]+>\s*(?:<tr><td>\s*<\/td><\/tr>\s*)*<tr><td>\s*<table[^>]+>\s*<tr>\s*(.+?)\s*<tr class="mep_CVtext"><td><br\/><\/td><\/tr>\s*<\/table>\s*(?:<\/td><\/tr>\s*<tr><td>\s*)*(?:<!-- START : PARLIAMENTARY WORKS  -->|<table[^>]+>\s*<tr>\s*<td[^>]+>Curriculum [Vv]itae<\/td>)/os);
        $functions_txt = "" unless $functions_txt;
        $functions_txt =~ s/\s\s+//go;
        $functions_txt =~ s/\n//go;
        $functions_txt =~ s/<\/?tr[^>]*>//go;
        $functions_txt =~ s/<\/?table[^>]*>//go;
        $functions_txt =~ s/<a[^>]+>[^<]+<\/a>//go;
        $functions_txt =~ s/<td><br\/><\/td>//go;
        $functions_txt =~ s/<td[^>]+"><img[^>]+><\/td>/ /go;
        $functions_txt =~ s/<\/?td>//go;
        $functions_txt =~ s/ <td/<td/go;
        my @functions_by_role = split /<td colspan="2" class="mepcountry">/o, $functions_txt;
        shift @functions_by_role;

        # Parse functions sorted by role
        my @unsorted_functions;
        foreach my $func_by_role (@functions_by_role) {
            $func_by_role =~ s/^(.+?)(?=<td)//o;
            my $role = $1;
            my @funcs = split /<td style="width: 98%;">/o, $func_by_role;
            shift @funcs;
            # Parse other infos: function name and period
            foreach my $func (@funcs) {
                $func =~ s/^(.+?) ://o;
                my $period = $1;
                my ($begin_day, $begin_month, $begin_year, $end_day, $end_month, $end_year) =
                    ($period =~ /^(\d\d)\.(\d\d)\.(\d\d\d\d) \/ (\d\d)\.(\d\d)\.(\d\d\d\d)$/);
                push @unsorted_functions, {
                                        ROLE        => $role,
                                        VALUE       => $func,
                                        BEGIN_DAY   => $begin_day,
                                        BEGIN_MONTH => $begin_month,
                                        BEGIN_YEAR  => $begin_year,
                                        END_DAY     => $end_day,
                                        END_MONTH   => $end_month,
                                        END_YEAR    => $end_year,
                                     };
            }
        }

        # Try to join adjacent periods
        my %roles_order = (
                            'Présidente'       => 1,
                            'Président'        => 2,
                            'Vice-présidente'  => 3,
                            'Vice-président'   => 4,
                            'Membre'           => 5,
                            'Membre suppléant' => 6,
                          );
        my @to_be_merged_functions = sort {
                                                      $a->{VALUE} cmp $b->{VALUE}
                                                                   ||
                                                 $a->{BEGIN_YEAR} <=> $b->{BEGIN_YEAR}
                                                                   ||
                                                $a->{BEGIN_MONTH} <=> $b->{BEGIN_MONTH}
                                                                   ||
                                                  $a->{BEGIN_DAY} <=> $b->{BEGIN_DAY}
                                                                   ||
                                                   $a->{END_YEAR} <=> $b->{END_YEAR}
                                                                   ||
                                                  $a->{END_MONTH} <=> $b->{END_MONTH}
                                                                   ||
                                                    $a->{END_DAY} <=> $b->{END_DAY}
                                                                   ||
                                        $roles_order{$a->{ROLE}}  <=> $roles_order{$b->{ROLE}}
                                    } @unsorted_functions;
        my @merged_functions = ();                            
        my $previous_function = { VALUE => 'init' };
        foreach my $function (@to_be_merged_functions) {
            if ($function->{VALUE} eq $previous_function->{VALUE} and
                $function->{ROLE} eq $previous_function->{ROLE} and
                Delta_Format(
                    DateCalc(
                        ParseDate("$previous_function->{END_MONTH}/$previous_function->{END_DAY}/$previous_function->{END_YEAR}"),
                        ParseDate("$function->{BEGIN_MONTH}/$function->{BEGIN_DAY}/$function->{BEGIN_YEAR}")
                    ), 'approx', 1, '%dt'
                ) <= 1
               ) {
                $previous_function->{END_YEAR} = $function->{END_YEAR};
                $previous_function->{END_MONTH} = $function->{END_MONTH};
                $previous_function->{END_DAY} = $function->{END_DAY};
            }
            else {
                push @merged_functions, $previous_function if $previous_function->{VALUE} ne 'init';
                $previous_function = $function;
            }
        }

        # Chronological sort
        my @functions = sort {
                                          $a->{BEGIN_YEAR} <=> $b->{BEGIN_YEAR}
                                                            ||
                                         $a->{BEGIN_MONTH} <=> $b->{BEGIN_MONTH}
                                                            ||
                                           $a->{BEGIN_DAY} <=> $b->{BEGIN_DAY}
                                                            ||
                                            $a->{END_YEAR} <=> $b->{END_YEAR}
                                                            ||
                                           $a->{END_MONTH} <=> $b->{END_MONTH}
                                                            ||
                                             $a->{END_DAY} <=> $b->{END_DAY}
                                                            ||
                                 $roles_order{$a->{ROLE}}  <=> $roles_order{$b->{ROLE}}
                             } @merged_functions;

        # Replace old functions by new ones
        my $functions_elt = $xml->root->first_child('functions') ? $xml->root->first_child('functions') : $xml->root->first_child('contact')->insert_new_elt('after','functions');
        $functions_elt->cut_children;
        foreach my $func (@functions) {
            my $function = $functions_elt->insert_new_elt('last_child', 'function');
            $function->insert_new_elt('last_child', 'label', $func->{VALUE});
            $function->insert_new_elt('last_child', 'role', $func->{ROLE});
            my $function_begin = $function->insert_new_elt('last_child', 'begin_term');
            $function_begin->insert_new_elt('last_child', 'day', $func->{BEGIN_DAY});
            $function_begin->insert_new_elt('last_child', 'month', $func->{BEGIN_MONTH});
            $function_begin->insert_new_elt('last_child', 'year', $func->{BEGIN_YEAR});
            my $function_end = $function->insert_new_elt('last_child', 'end_term');
            $function_end->insert_new_elt('last_child', 'day', $func->{END_DAY});
            $function_end->insert_new_elt('last_child', 'month', $func->{END_MONTH});
            $function_end->insert_new_elt('last_child', 'year', $func->{END_YEAR});
        }
    }
    else {
        warn "Error fetching $outgoing_url: ", $outgoing_res->status_line, "\n";
    }

    # Update document into database
    $docString = $xml->sprint;
    $theDocument->setContent($docString);
    $container->updateDocument($theDocument, $updateContext);
    print "$docName updated in container $theContainer\n";

    exit 0;
};

if (my $e = catch std::exception) {
    warn "Exception:\n";
    warn $e->what() . "\n";
    exit( -1 );
}
elsif ($@) {
    warn "Query failed\n";
    warn $@;
    exit( -1 );
}
